# -*- coding: utf-8 -*-
from plone.app.widgets.interfaces import IFieldPermissionChecker
from Products.Archetypes.atapi import listTypes
from Products.Archetypes.interfaces import IBaseObject
from Products.ATContentTypes.config import PROJECTNAME
from Products.ATContentTypes.interfaces import IATTopic
from Products.ATContentTypes.interfaces import IATTopicCriterion
from Products.CMFCore.permissions import setDefaultRoles
from zope.component import adapts
from zope.interface import implements


TYPE_ROLES = ('Manager', 'Site Administrator', 'Owner')
TOPIC_ROLES = ('Manager', 'Site Administrator')
CHANGE_TOPIC_ROLES = TOPIC_ROLES + ('Owner',)
CRITERION_ROLES = ('Manager', 'Site Administrator')

# Gathering Topic and Event related permissions into one place
AddTopics = 'Add portal topics'
setDefaultRoles(AddTopics, TOPIC_ROLES)

ChangeTopics = 'Change portal topics'
setDefaultRoles(ChangeTopics, CHANGE_TOPIC_ROLES)

ChangeEvents = 'Change portal events'
setDefaultRoles(ChangeEvents, ('Manager', 'Site Administrator', 'Owner',))

ModifyConstrainTypes = "Modify constrain types"
setDefaultRoles(ModifyConstrainTypes,
                ('Manager', 'Site Administrator', 'Owner'))

ModifyViewTemplate = "Modify view template"
setDefaultRoles(ModifyViewTemplate, ('Manager', 'Site Administrator', 'Owner'))

ViewHistory = "ATContentTypes: View history"
setDefaultRoles(ViewHistory, ('Manager', 'Site Administrator'))

UploadViaURL = "ATContentTypes: Upload via url"
setDefaultRoles(UploadViaURL, ('Manager', 'Site Administrator'))

permissions = {}


def wireAddPermissions():
    """Creates a list of add permissions for all types in this project

    Must be called **after** all types are registered!
    """
    global permissions
    atct_types = listTypes(PROJECTNAME)
    for atct in atct_types:
        if IATTopic.implementedBy(atct['klass']):
            permission = AddTopics
        elif IATTopicCriterion.implementedBy(atct['klass']):
            permission = "%s Topic: Add %s" % (
                PROJECTNAME, atct['portal_type'])
            setDefaultRoles(permission, CRITERION_ROLES)
        else:
            permission = "%s: Add %s" % (PROJECTNAME, atct['portal_type'])
            setDefaultRoles(permission, TYPE_ROLES)

        permissions[atct['portal_type']] = permission
    return permissions


class ATFieldPermissionChecker(object):
    implements(IFieldPermissionChecker)
    adapts(IBaseObject)

    def __init__(self, context):
        self.context = context

    def validate(self, field_name, vocabulary_name=None):
        field = self.context.getField(field_name)
        if field is not None:
            # If a vocabulary name was specified and it doesn't match
            # the value for the field or the widget, fail.
            if vocabulary_name and (
               vocabulary_name != getattr(field.widget, 'vocabulary', None) and
               vocabulary_name != getattr(field, 'vocabulary_factory', None)):
                return False
            return field.checkPermission('w', self.context)
        raise AttributeError('No such field: {}'.format(field_name))



## Script (Python) "link_redirect_view"
##bind container=container
##bind context=context
##bind namespace=
##bind script=script
##bind subpath=traverse_subpath
##parameters=

"""
Redirect to the Link target URL, if and only if:
 - redirect_links property is enabled in portal_properties/site_properties
 - AND current user doesn't have permission to edit the Link
"""

from Products.CMFCore.utils import getToolByName


mtool = getToolByName(context, 'portal_membership')

redirect_links = context.portal_registry['plone.redirect_links']
can_edit = mtool.checkPermission('Modify portal content', context)

if redirect_links and not can_edit:
    if context.getRemoteUrl().startswith('.'):
        # we just need to adapt ../relative/links, /absolute/ones work anyway
        # -> this requires relative links to start with ./ or ../
        context_state = context.restrictedTraverse('@@plone_context_state')
        return context.REQUEST.RESPONSE.redirect(
                    context_state.canonical_object_url()
                    + '/'
                    + context.getRemoteUrl())
    else:
        return context.REQUEST.RESPONSE.redirect(context.getRemoteUrl())
else:
    # link_view.pt is a template in the plone_content skin layer
    return context.link_view()



# -*- coding: utf-8 -*-
from plone.app.testing.bbb import PloneTestCase
from Products.ATContentTypes import setuphandlers


# No sense to check this in Plone 5, frontpage is DX
class TestContentProfile(PloneTestCase):

    def afterSetUp(self):
        portal_setup = self.portal.portal_setup
        portal_setup.runAllImportStepsFromProfile(
            'profile-Products.ATContentTypes:content')

    def testPortalContentLanguage(self):
        from zope.component import provideUtility
        from zope.i18n.interfaces import ITranslationDomain
        from zope.i18n.simpletranslationdomain import SimpleTranslationDomain

        # Let's fake the news title translations
        messages = {
            ('de', u'news-title'): u'Foo',
            ('pt_BR', u'news-title'): u'Bar',
        }
        pfp = SimpleTranslationDomain('plonefrontpage', messages)
        provideUtility(pfp, ITranslationDomain, name='plonefrontpage')

        # Setup the new placeholder folders
        self.folder.invokeFactory('Folder', 'brazilian')
        self.folder.invokeFactory('Folder', 'german')

        # Check if the content is being created in German
        self.folder.german.setLanguage('de')
        self.loginAsPortalOwner()
        setuphandlers.setupPortalContent(self.folder.german)
        # self.assertEqual(self.folder.german.news.Title(), 'Foo')

        # Check if the content is being created in a composite
        # language code, in this case Brazilian Portuguese
        self.folder.brazilian.setLanguage('pt-br')
        setuphandlers.setupPortalContent(self.folder.brazilian)
        # self.assertEqual(self.folder.brazilian.news.Title(), 'Bar')



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from AccessControl import getSecurityManager
from AccessControl import Owned
from Acquisition import aq_base
from Acquisition import aq_get
from Acquisition import aq_inner
from Acquisition import aq_parent
from App.class_init import InitializeClass
from App.Common import package_home
from OFS.SimpleItem import SimpleItem
from Products.ATContentTypes.config import GLOBALS
from Products.ATContentTypes.interfaces import IFactoryTool
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.utils import UniqueObject
from Products.CMFPlone.interfaces import IHideFromBreadcrumbs
from Products.CMFPlone.PloneBaseTool import PloneBaseTool
from Products.CMFPlone.PloneFolder import PloneFolder as TempFolderBase
from Products.CMFPlone.utils import base_hasattr
from Products.CMFPlone.utils import log_exc
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from zExceptions import NotFound
from ZODB.POSException import ConflictError
from zope.interface import implements
from zope.structuredtext import stx2html
from ZPublisher.mapply import mapply
from ZPublisher.Publish import call_object
from ZPublisher.Publish import dont_publish_class
from ZPublisher.Publish import missing_name

import logging
import os


FACTORY_INFO = '__factory__info__'


class FauxArchetypeTool(object):
    """A faux archetypes tool which prevents content from being indexed."""

    __allow_access_to_unprotected_subobjects__ = 1

    def __init__(self, tool):
        self.tool = tool

    def getCatalogsByType(self, type_name):
        return []

    def __getitem__(self, id):
        return getattr(self.tool, id)


def _createObjectByType(type_name, container, id, *args, **kw):
    """This function replaces Products.CMFPlone.utils._createObjectByType.

    If no product is set on fti, use IFactory to lookup the factory.
    Additionally we add 'container' as 'parent' kw argument when calling the
    IFactory implementation. this ensures the availability of the acquisition
    chain if needed inside the construction logic.

    The kw argument hack is some kind of semi-valid since the IFactory
    interface promises the __call__ function to accept all given args and kw
    args. As long as the specific IFactory implementation provides this
    signature everything works well unless any other 3rd party factory expects
    another kind of object as 'parent' kw arg than the provided one.
    """
    id = str(id)
    typesTool = getToolByName(container, 'portal_types')
    fti = typesTool.getTypeInfo(type_name)
    if not fti:
        raise ValueError('Invalid type %s' % type_name)

    if not fti.product:
        kw['parent'] = container

    return fti._constructInstance(container, id, *args, **kw)


# #############################################################################
# A class used for generating the temporary folder that will
# hold temporary objects.  We need a separate class so that
# we can add all types to types_tool's allowed_content_types
# for the class without having side effects in the rest of
# the portal.
class TempFolder(TempFolderBase):

    portal_type = meta_type = 'TempFolder'
    isPrincipiaFolderish = 0

    implements(IHideFromBreadcrumbs)

    # override getPhysicalPath so that temporary objects return a full path
    # that includes the acquisition parent of portal_factory (otherwise we get
    # portal_root/portal_factory/... no matter where the object will reside)
    def getPhysicalPath(self):
        '''Returns a path (an immutable sequence of strings)
        that can be used to access this object again
        later, for example in a copy/paste operation.  getPhysicalRoot()
        and getPhysicalPath() are designed to operate together.
        '''
        portal_factory = aq_parent(aq_inner(self))
        path = aq_parent(portal_factory).getPhysicalPath() + \
            (portal_factory.getId(), self.getId(), )
        return path

    # override / delegate local roles methods
    def __ac_local_roles__(self):
        """__ac_local_roles__ needs to be handled carefully.
        Zope's and GRUF's User.getRolesInContext both walk up the
        acquisition hierarchy using aq_parent(aq_inner(obj)) when
        they gather local roles, and this process will result in
        their walking from TempFolder to portal_factory to the portal root."""
        object = aq_parent(aq_parent(self))
        local_roles = {}
        while True:
            # Get local roles for this user
            lr = getattr(object, '__ac_local_roles__', None)
            if lr:
                if callable(lr):
                    lr = lr()
                lr = lr or {}
                for k, v in lr.items():
                    if k not in local_roles:
                        local_roles[k] = []
                    for role in v:
                        if role not in local_roles[k]:
                            local_roles[k].append(role)

            # Check if local role has to be acquired (PLIP 16)
            if getattr(object, '__ac_local_roles_block__', None):
                # Ok, we have to stop there, as lr. blocking is enabled
                break

            # Prepare next iteration
            inner = getattr(object, 'aq_inner', object)
            parent = getattr(inner, 'aq_parent', None)
            if parent is not None:
                object = parent
                continue
            if hasattr(object, 'im_self'):
                object = object.im_self
                object = getattr(object, 'aq_inner', object)
                continue
            break
        return local_roles

    def has_local_roles(self):
        return len(self.__ac_local_roles__())

    def get_local_roles_for_userid(self, userid):
        return tuple(self.__ac_local_roles__().get(userid, []))

    def get_valid_userids(self):
        return aq_parent(aq_parent(self)).get_valid_userids()

    def valid_roles(self):
        return aq_parent(aq_parent(self)).valid_roles()

    def validate_roles(self, roles):
        return aq_parent(aq_parent(self)).validate_roles(roles)

    def userdefined_roles(self):
        return aq_parent(aq_parent(self)).userdefined_roles()

    # delegate Owned methods
    def owner_info(self):
        return aq_parent(aq_parent(self)).owner_info()

    def getOwner(self, info=0,
                 aq_get=aq_get,
                 UnownableOwner=Owned.UnownableOwner,
                 getSecurityManager=getSecurityManager,
                 ):
        return aq_parent(
            aq_parent(self)).getOwner(
            info, aq_get, UnownableOwner, getSecurityManager)

    def userCanTakeOwnership(self):
        return aq_parent(aq_parent(self)).userCanTakeOwnership()

    # delegate allowedContentTypes
    def allowedContentTypes(self):
        return aq_parent(aq_parent(self)).allowedContentTypes()

    def __getitem__(self, id):
        # Zope's inner acquisition chain for objects returned by __getitem__
        # will be:
        #
        # portal -> portal_factory -> temporary_folder -> object
        #
        # What we really want is for the inner acquisition chain to be:
        #
        # intended_parent_folder -> portal_factory -> temporary_folder ->
        # object
        #
        # So we need to rewrap...
        portal_factory = aq_parent(aq_inner(self))
        intended_parent = aq_parent(portal_factory)

        # If the intended parent has an object with the given id, just do a
        # passthrough
        if hasattr(intended_parent, id):
            return getattr(intended_parent, id)

        # rewrap portal_factory
        portal_factory = aq_base(portal_factory).__of__(intended_parent)
        # rewrap self
        temp_folder = aq_base(self).__of__(portal_factory)

        if id in self:
            return (aq_base(self._getOb(id)).__of__(temp_folder)) \
                .__of__(intended_parent)
        else:
            type_name = self.getId()
            try:
                # We fake an archetype tool which returns no catalogs for the
                # object to be indexed in to avoid it showing up in the catalog
                # in the first place.
                self.archetype_tool = FauxArchetypeTool(
                    getToolByName(self, 'archetype_tool'))
                _createObjectByType(type_name, self, id)
            except ConflictError:
                raise
            except:
                # some errors from invokeFactory (AttributeError, maybe others)
                # get swallowed -- dump the exception to the log to make sure
                # developers can see what's going on
                log_exc(severity=logging.DEBUG)
                raise
            obj = self._getOb(id)

            # keep obj out of the catalog
            obj.unindexObject()

            # additionally keep it out of Archetypes UID and refs catalogs
            if base_hasattr(obj, '_uncatalogUID'):
                obj._uncatalogUID(obj)
            if base_hasattr(obj, '_uncatalogRefs'):
                obj._uncatalogRefs(obj)

            return (aq_base(obj).__of__(temp_folder)).__of__(intended_parent)

    # ignore rename requests since they don't do anything
    def manage_renameObject(self, id, new_id, REQUEST=None):
        pass


# #############################################################################
class FactoryTool(PloneBaseTool, UniqueObject, SimpleItem):
    """ """
    id = 'portal_factory'
    meta_type = 'Plone Factory Tool'
    toolicon = 'skins/plone_images/add_icon.png'
    security = ClassSecurityInfo()
    isPrincipiaFolderish = 0

    implements(IFactoryTool, IHideFromBreadcrumbs)

    manage_options = (
        ({'label': 'Overview', 'action': 'manage_overview'},
         {'label': 'Documentation', 'action': 'manage_docs'},
         {'label': 'Factory Types', 'action': 'manage_portal_factory_types'}) +
        SimpleItem.manage_options)

    wwwpath = os.path.join(package_home(GLOBALS), 'www')

    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = PageTemplateFile(
        os.path.join(wwwpath, 'portal_factory_manage_overview'), globals())
    manage_overview.__name__ = 'manage_overview'
    manage_overview._need__name__ = 0

    security.declareProtected(ManagePortal, 'manage_portal_factory_types')
    manage_portal_factory_types = PageTemplateFile(
        os.path.join(wwwpath, 'portal_factory_manage_types'), globals())
    manage_portal_factory_types.__name__ = 'manage_portal_factory_types'
    manage_portal_factory_types._need__name__ = 0

    manage_main = manage_overview

    security.declareProtected(ManagePortal, 'manage_docs')
    manage_docs = PageTemplateFile(
        os.path.join(wwwpath, 'portal_factory_manage_docs'), globals())
    manage_docs.__name__ = 'manage_docs'

    f = open(os.path.join(wwwpath, 'portal_factory_docs.stx'), 'r')
    _docs = f.read()
    f.close()
    _docs = stx2html(_docs)

    @security.public
    def docs(self):
        """Returns FactoryTool docs formatted as HTML"""
        return self._docs

    def getFactoryTypes(self):
        if not hasattr(self, '_factory_types'):
            self._factory_types = {}
        return self._factory_types

    @security.protected(ManagePortal)
    def manage_setPortalFactoryTypes(self, REQUEST=None, listOfTypeIds=None):
        """Set the portal types that should use the factory."""
        if listOfTypeIds is not None:
            dict = {}
            for l in listOfTypeIds:
                dict[l] = 1
        elif REQUEST is not None:
            dict = REQUEST.form
        if dict is None:
            dict = {}
        self._factory_types = {}
        types_tool = getToolByName(self, 'portal_types')
        for t in types_tool.listContentTypes():
            if t in dict:
                self._factory_types[t] = 1
        self._p_changed = 1
        if REQUEST:
            REQUEST.RESPONSE.redirect('manage_main')

    def doCreate(self, obj, id=None, **kw):
        """Create a real object from a temporary object."""
        if self.isTemporary(obj=obj):
            if id is not None:
                id = id.strip()
            if not id:
                if hasattr(obj, 'getId') and callable(getattr(obj, 'getId')):
                    id = obj.getId()
                else:
                    id = getattr(obj, 'id', None)
            # get the ID of the TempFolder
            type_name = aq_parent(aq_inner(obj)).id
            folder = aq_parent(aq_parent(aq_parent(aq_inner(obj))))
            folder.invokeFactory(id=id, type_name=type_name)
            obj = getattr(folder, id)

            # give ownership to currently authenticated member if not anonymous
            # TODO is this necessary?
            membership_tool = getToolByName(self, 'portal_membership')
            if not membership_tool.isAnonymousUser():
                member = membership_tool.getAuthenticatedMember()
                obj.changeOwnership(member.getUser(), 1)
            if hasattr(aq_base(obj), 'manage_afterPortalFactoryCreate'):
                obj.manage_afterPortalFactoryCreate()
        return obj

    def _fixRequest(self):
        """Our before_publishing_traverse call mangles URL0.  This fixes up
        the REQUEST."""
        # Everything seems to work without this method being called at all...
        factory_info = self.REQUEST.get(FACTORY_INFO, None)
        if not factory_info:
            return
        stack = factory_info['stack']
        FACTORY_URL = self.REQUEST.URL
        URL = '/'.join([FACTORY_URL] + stack)
        self.REQUEST.set('URL', URL)

        url_list = URL.split('/')
        n = 0
        while len(url_list) > 0 and url_list[-1] != '':
            self.REQUEST.set('URL%d' % n, '/'.join(url_list))
            url_list = url_list[:-1]
            n = n + 1

        # BASE1 is the url of the Zope App object
        n = len(self.REQUEST._steps) + 2
        base = FACTORY_URL
        for part in stack:
            base = '%s/%s' % (base, part)
            self.REQUEST.set('BASE%d' % n, base)
            n += 1
        # TODO fix URLPATHn, BASEPATHn here too

    def isTemporary(self, obj):
        """Check to see if an object is temporary"""
        ob = aq_base(aq_parent(aq_inner(obj)))
        return hasattr(ob, 'meta_type') \
            and ob.meta_type == TempFolder.meta_type

    def __before_publishing_traverse__(self, other, REQUEST):

        if REQUEST.get(FACTORY_INFO, None):
            del REQUEST[FACTORY_INFO]

        stack = REQUEST.get('TraversalRequestNameStack')
        # convert from unicode if necessary (happens in Epoz for some weird
        # reason)
        stack = [str(s) for s in stack]

        # need 2 more things on the stack at least for portal_factory to
        # kick in:
        #    (1) a type, and (2) an id
        if len(stack) < 2:  # ignore
            return

        # Keep track of how many path elements we want to eat
        gobbled_length = 0

        type_name = stack[-1]
        types_tool = getToolByName(self, 'portal_types')
        # make sure this is really a type name
        if type_name not in types_tool.listContentTypes():
            return  # nope -- do nothing

        gobbled_length += 1

        id = stack[-2]
        intended_parent = aq_parent(self)
        if hasattr(intended_parent, id):
            return  # do normal traversal via __bobo_traverse__

        gobbled_length += 1

        # about to create an object

        # before halting traversal, check for method aliases
        # stack should be [...optional stuff..., id, type_name]
        key = len(stack) >= 3 and stack[-3] or '(Default)'
        ti = types_tool.getTypeInfo(type_name)
        method_id = ti and ti.queryMethodID(key)
        if method_id:
            if key != '(Default)':
                del(stack[-3])
            if method_id != '(Default)':
                stack.insert(-2, method_id)
                gobbled_length += 1
            REQUEST._hacked_path = 1
        else:
            gobbled_length += 1

        # Pevent further traversal if we are doing a normal factory request,
        # but allow it if there is a traversal sub-path beyond the (edit)
        # view on the content item. In this case, portal_factory will not
        # be responsible for rendering the object.
        if len(stack) <= gobbled_length:
            REQUEST.set('TraversalRequestNameStack', [])

        stack.reverse()
        factory_info = {'stack': stack}
        REQUEST.set(FACTORY_INFO, factory_info)

    def __bobo_traverse__(self, REQUEST, name):
        # __bobo_traverse__ can be invoked directly by a restricted_traverse
        # method call in which case the traversal stack will not have been
        # cleared by __before_publishing_traverse__
        name = str(name)  # fix unicode weirdness
        types_tool = getToolByName(self, 'portal_types')
        if name not in types_tool.listContentTypes():
            # not a type name -- do the standard thing
            return getattr(self, name)
        # a type name -- return a temp folder
        return self._getTempFolder(str(name))

    @security.public
    def __call__(self, *args, **kwargs):
        """call method"""
        self._fixRequest()
        factory_info = self.REQUEST.get(FACTORY_INFO, {})
        stack = factory_info['stack']
        type_name = stack[0]
        id = stack[1]

        # do a passthrough if parent contains the id
        if id in aq_parent(self):
            return aq_parent(self).restrictedTraverse(
                '/'.join(stack[1:]))(*args, **kwargs)

        tempFolder = self._getTempFolder(type_name)
        # Mysterious hack that fixes some problematic interactions with
        # SpeedPack:
        #   Get the first item in the stack by explicitly calling __getitem__
        temp_obj = tempFolder.__getitem__(id)
        stack = stack[2:]
        if stack:
            try:
                obj = temp_obj.restrictedTraverse('/'.join(stack))
            except AttributeError:
                raise NotFound

            # Mimic URL traversal, sort of
            if getattr(aq_base(obj), 'index_html', None):
                obj = obj.restrictedTraverse('index_html')
            else:
                obj = getattr(obj, 'GET', obj)

        else:
            obj = temp_obj
        return mapply(obj, self.REQUEST.args, self.REQUEST,
                      call_object, 1, missing_name,
                      dont_publish_class, self.REQUEST, bind=1)

    index_html = None  # call __call__, not index_html

    def _getTempFolder(self, type_name):
        factory_info = self.REQUEST.get(FACTORY_INFO, {})
        tempFolder = factory_info.get(type_name, None)
        if tempFolder is not None:
            tempFolder = aq_inner(tempFolder).__of__(self)
            return tempFolder

        # make sure we can add an object of this type to the temp folder
        types_tool = getToolByName(self, 'portal_types')
        if type_name not in types_tool.TempFolder.allowed_content_types:
            # update allowed types for tempfolder
            types_tool.TempFolder.allowed_content_types = \
                (types_tool.listContentTypes())

        tempFolder = TempFolder(type_name).__of__(self)

        factory_info[type_name] = tempFolder
        self.REQUEST.set(FACTORY_INFO, factory_info)
        return tempFolder

InitializeClass(FactoryTool)



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from plone.i18n.normalizer.interfaces import IURLNormalizer
from plone.portlets.interfaces import ILocalPortletAssignmentManager
from plone.portlets.interfaces import IPortletManager
from plone.registry.interfaces import IRegistry
from Products.ATContentTypes.lib import constraintypes
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces import ILanguageSchema
from Products.CMFPlone.utils import _createObjectByType
from Products.CMFPlone.utils import bodyfinder
from zope.component import getUtility
from zope.component import queryMultiAdapter
from zope.component import queryUtility
from zope.i18n.interfaces import ITranslationDomain
from zope.i18n.locales import locales


def assignTitles(portal):
    ''' Check for those objects inside portal.
    If they are found we assign title to them.
    '''
    titles = {
        'portal_atct': 'Collection and image scales settings',
        'portal_factory': 'Responsible for the creation of content objects',
        'portal_form_controller': 'Registration of form and validation chains',
        'portal_metadata': 'Controls metadata like keywords, copyrights, etc',
    }
    for oid, obj in portal.items():
        title = titles.get(oid, None)
        if title:
            setattr(aq_base(obj), 'title', title)


def setupPortalContent(p):
    """
    Import default plone content
    """
    existing = p.keys()
    wftool = getToolByName(p, "portal_workflow")

    reg = queryUtility(IRegistry, context=p)
    language = reg['plone.default_language']

    # language = p.Language()
    parts = (language.split('-') + [None, None])[:3]
    locale = locales.getLocale(*parts)
    target_language = base_language = locale.id.language

    # If we get a territory, we enable the combined language codes
    use_combined = False
    if locale.id.territory:
        use_combined = True
        target_language += '_' + locale.id.territory

    # As we have a sensible language code set now, we disable the
    # start neutral functionality
    pprop = getToolByName(p, "portal_properties")
    sheet = pprop.site_properties

    registry = getUtility(IRegistry)
    language_settings = registry.forInterface(
        ILanguageSchema,
        prefix='plone'
    )
    language_settings.use_combined_language_codes = use_combined
    language_settings.default_language = language
    language_settings.available_languages = [language]

    # Enable visible_ids for non-latin scripts

    # See if we have an url normalizer
    normalizer = queryUtility(IURLNormalizer, name=target_language)
    if normalizer is None:
        normalizer = queryUtility(IURLNormalizer, name=base_language)

    # If we get a script other than Latn we enable visible_ids
    if locale.id.script is not None:
        if locale.id.script.lower() != 'latn':
            sheet.visible_ids = True

    # If we have a normalizer it is safe to disable the visible ids
    if normalizer is not None:
        sheet.visible_ids = False

    request = getattr(p, 'REQUEST', None)
    # The front-page
    if 'front-page' not in existing:
        front_title = u'Welcome to Plone'
        front_desc = u'Congratulations! You have successfully installed Plone.'
        front_text = None
        _createObjectByType('Document', p, id='front-page',
                            title=front_title, description=front_desc)
        fp = p['front-page']
        if wftool.getInfoFor(fp, 'review_state') != 'published':
            wftool.doActionFor(fp, 'publish')

        if base_language != 'en':
            util = queryUtility(ITranslationDomain, 'plonefrontpage')
            if util is not None:
                front_title = util.translate(
                    u'front-title',
                    target_language=target_language,
                    default="Welcome to Plone")
                front_desc = util.translate(
                    u'front-description',
                    target_language=target_language,
                    default="Congratulations! You have successfully installed "
                            "Plone.")
                translated_text = util.translate(
                    u'front-text', target_language=target_language)
                if translated_text != u'front-text':
                    front_text = translated_text

        if front_text is None and request is not None:
            view = queryMultiAdapter((p, request),
                                     name='plone-frontpage-setup')
            if view is not None:
                front_text = bodyfinder(view.index()).strip()

        fp.setTitle(front_title)
        fp.setDescription(front_desc)
        fp.setLanguage(language)
        fp.setText(front_text, mimetype='text/html')

        # Show off presentation mode
        if hasattr(fp, 'setPresentation'):
            fp.setPresentation(True)

        # Mark as fully created
        fp.unmarkCreationFlag()

        p.setDefaultPage('front-page')
        fp.reindexObject()

    # News topic
    if 'news' not in existing:
        news_title = 'News'
        news_desc = 'Site News'
        if base_language != 'en':
            util = queryUtility(ITranslationDomain, 'plonefrontpage')
            if util is not None:
                news_title = util.translate(u'news-title',
                                            target_language=target_language,
                                            default='News')
                news_desc = util.translate(u'news-description',
                                           target_language=target_language,
                                           default='Site News')

        _createObjectByType('Folder', p, id='news',
                            title=news_title, description=news_desc)
        _createObjectByType('Collection', p.news, id='aggregator',
                            title=news_title, description=news_desc)

        folder = p.news
        folder.setOrdering('unordered')
        folder.setConstrainTypesMode(constraintypes.ENABLED)
        folder.setLocallyAllowedTypes(['News Item'])
        folder.setImmediatelyAddableTypes(['News Item'])
        folder.setDefaultPage('aggregator')
        folder.unmarkCreationFlag()
        folder.setLanguage(language)

        if wftool.getInfoFor(folder, 'review_state') != 'published':
            wftool.doActionFor(folder, 'publish')

        topic = p.news.aggregator
        topic.setLanguage(language)

        query = [{'i': 'portal_type',
                  'o': 'plone.app.querystring.operation.selection.any',
                  'v': ['News Item']},
                 {'i': 'review_state',
                  'o': 'plone.app.querystring.operation.selection.any',
                  'v': ['published']}]
        topic.setQuery(query)

        topic.setSort_on('effective')
        topic.setSort_reversed(True)
        topic.setLayout('folder_summary_view')
        topic.unmarkCreationFlag()

        if wftool.getInfoFor(topic, 'review_state') != 'published':
            wftool.doActionFor(topic, 'publish')

    # Events topic
    if 'events' not in existing:
        events_title = 'Events'
        events_desc = 'Site Events'
        if base_language != 'en':
            util = queryUtility(ITranslationDomain, 'plonefrontpage')
            if util is not None:
                events_title = util.translate(u'events-title',
                                              target_language=target_language,
                                              default='Events')
                events_desc = util.translate(u'events-description',
                                             target_language=target_language,
                                             default='Site Events')

        _createObjectByType('Folder', p, id='events',
                            title=events_title, description=events_desc)
        _createObjectByType('Collection', p.events, id='aggregator',
                            title=events_title, description=events_desc)
        folder = p.events
        folder.setOrdering('unordered')
        folder.setConstrainTypesMode(constraintypes.ENABLED)
        folder.setLocallyAllowedTypes(['Event'])
        folder.setImmediatelyAddableTypes(['Event'])
        folder.setDefaultPage('aggregator')
        folder.unmarkCreationFlag()
        folder.setLanguage(language)

        if wftool.getInfoFor(folder, 'review_state') != 'published':
            wftool.doActionFor(folder, 'publish')

        topic = folder.aggregator
        topic.unmarkCreationFlag()
        topic.setLanguage(language)

        query = [{'i': 'portal_type',
                  'o': 'plone.app.querystring.operation.selection.any',
                  'v': ['Event']},
                 {'i': 'start',
                  'o': 'plone.app.querystring.operation.date.afterToday',
                  'v': ''},
                 {'i': 'review_state',
                  'o': 'plone.app.querystring.operation.selection.any',
                  'v': ['published']}]
        topic.setQuery(query)
        topic.setSort_on('start')
    else:
        topic = p.events

    if wftool.getInfoFor(topic, 'review_state') != 'published':
        wftool.doActionFor(topic, 'publish')

    # configure Members folder
    members_title = 'Users'
    members_desc = "Site Users"
    if 'Members' not in existing:
        _createObjectByType('Folder', p, id='Members',
                            title=members_title, description=members_desc)

    if 'Members' in p.keys():
        if base_language != 'en':
            util = queryUtility(ITranslationDomain, 'plonefrontpage')
            if util is not None:
                members_title = util.translate(u'members-title',
                                               target_language=target_language,
                                               default='Users')
                members_desc = util.translate(u'members-description',
                                              target_language=target_language,
                                              default="Site Users")

        members = getattr(p, 'Members')
        members.setTitle(members_title)
        members.setDescription(members_desc)
        members.setOrdering('unordered')
        members.unmarkCreationFlag()
        members.setLanguage(language)
        members.reindexObject()

        if wftool.getInfoFor(members, 'review_state') != 'published':
            wftool.doActionFor(members, 'publish')

        members.layout = '@@member-search'

        # Block all right column portlets by default
        manager = queryUtility(IPortletManager, name='plone.rightcolumn')
        if manager is not None:
            assignable = queryMultiAdapter(
                (members, manager),
                ILocalPortletAssignmentManager)
            assignable.setBlacklistStatus('context', True)
            assignable.setBlacklistStatus('group', True)
            assignable.setBlacklistStatus('content_type', True)


def importContent(context):
    # Only run step if a flag file is present
    if context.readDataFile('atct-content.txt') is None:
        return
    site = context.getSite()
    setupPortalContent(site)
    assignTitles(site)



# -*- coding: utf-8 -*-
from Products.ATContentTypes.content import link

import logging


logger = logging.getLogger('ATContentTypes')

logger.warn(
    'You have content instances of the ATFavorite type in your site. '
    'This type is no longer available. Please remove all remaining instances '
    'of this type or migrate them to links.')


class ATFavorite(link.ATLink):
    """BBB: The old and long removed favorite content type."""

    portal_type = 'Favorite'
    archetype_name = 'Favorite'



# -*- coding: utf-8 -*-
from Products.ATContentTypes.configuration.config import zconf  # noqa
from Products.ATContentTypes.configuration.config import handler  # noqa
from Products.ATContentTypes.configuration.config import conf_file  # noqa



# -*- coding: utf-8 -*-
from Products.ATContentTypes.content.base import ATCTBTreeFolder  # noqa
from Products.ATContentTypes.content.base import ATCTContent  # noqa
from Products.ATContentTypes.content.base import ATCTFileContent  # noqa
from Products.ATContentTypes.content.base import ATCTFolder  # noqa
from Products.ATContentTypes.content.base import ATCTOrderedFolder  # noqa
from Products.ATContentTypes.content.document import ATDocument  # noqa
from Products.ATContentTypes.content.document import ATDocumentSchema  # noqa
from Products.ATContentTypes.content.event import ATEvent  # noqa
from Products.ATContentTypes.content.event import ATEventSchema  # noqa
from Products.ATContentTypes.content.file import ATFile  # noqa
from Products.ATContentTypes.content.file import ATFileSchema  # noqa
from Products.ATContentTypes.content.folder import ATBTreeFolder  # noqa
from Products.ATContentTypes.content.folder import ATBTreeFolderSchema  # noqa
from Products.ATContentTypes.content.folder import ATFolder  # noqa
from Products.ATContentTypes.content.folder import ATFolderSchema  # noqa
from Products.ATContentTypes.content.image import ATImage  # noqa
from Products.ATContentTypes.content.image import ATImageSchema  # noqa
from Products.ATContentTypes.content.link import ATLink  # noqa
from Products.ATContentTypes.content.link import ATLinkSchema  # noqa
from Products.ATContentTypes.content.newsitem import ATNewsItem  # noqa
from Products.ATContentTypes.content.newsitem import ATNewsItemSchema  # noqa
from Products.ATContentTypes.content.schemata import ATContentTypeSchema  # noqa
from Products.ATContentTypes.content.topic import ATTopic  # noqa
from Products.ATContentTypes.content.topic import ATTopicSchema  # noqa
from Products.ATContentTypes.criteria import ALL_INDICES  # noqa
from Products.ATContentTypes.criteria import DATE_INDICES  # noqa
from Products.ATContentTypes.criteria import LIST_INDICES  # noqa
from Products.ATContentTypes.criteria import registerCriterion  # noqa
from Products.ATContentTypes.criteria import SORT_INDICES  # noqa
from Products.ATContentTypes.criteria import STRING_INDICES  # noqa
from Products.ATContentTypes.criteria import unregisterCriterion  # noqa
from Products.ATContentTypes.criteria.base import ATBaseCriterion  # noqa
from Products.ATContentTypes.criteria.boolean import ATBooleanCriterion  # noqa
from Products.ATContentTypes.criteria.boolean import ATBooleanCriterionSchema  # noqa
from Products.ATContentTypes.criteria.date import ATDateCriteria  # noqa
from Products.ATContentTypes.criteria.date import ATDateCriteriaSchema  # noqa
from Products.ATContentTypes.criteria.daterange import ATDateRangeCriterion  # noqa
from Products.ATContentTypes.criteria.daterange import ATDateRangeCriterionSchema  # noqa
from Products.ATContentTypes.criteria.list import ATListCriterion  # noqa
from Products.ATContentTypes.criteria.list import ATListCriterionSchema  # noqa
from Products.ATContentTypes.criteria.portaltype import ATPortalTypeCriterion  # noqa
from Products.ATContentTypes.criteria.portaltype import ATPortalTypeCriterionSchema  # noqa
from Products.ATContentTypes.criteria.reference import ATReferenceCriterion  # noqa
from Products.ATContentTypes.criteria.reference import ATReferenceCriterionSchema  # noqa
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema  # noqa
from Products.ATContentTypes.criteria.selection import ATSelectionCriterion  # noqa
from Products.ATContentTypes.criteria.selection import ATSelectionCriterionSchema  # noqa
from Products.ATContentTypes.criteria.simpleint import ATSimpleIntCriterion  # noqa
from Products.ATContentTypes.criteria.simpleint import ATSimpleIntCriterionSchema  # noqa
from Products.ATContentTypes.criteria.simplestring import ATSimpleStringCriterion  # noqa
from Products.ATContentTypes.criteria.simplestring import ATSimpleStringCriterionSchema  # noqa
from Products.ATContentTypes.criteria.sort import ATSortCriterion  # noqa
from Products.ATContentTypes.criteria.sort import ATSortCriterionSchema  # noqa



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from Products.ATContentTypes.config import HAS_MX_TIDY
from Products.ATContentTypes.config import MX_TIDY_ENABLED
from Products.ATContentTypes.config import MX_TIDY_MIMETYPES
from Products.ATContentTypes.config import MX_TIDY_OPTIONS
from Products.PageTemplates.Expressions import getEngine
from Products.validation.config import validation
from Products.validation.interfaces.IValidator import IValidator
from types import FileType
from zope.interface import implements
from zope.tal.htmltalparser import HTMLTALParser
from zope.tal.talgenerator import TALGenerator
from ZPublisher.HTTPRequest import FileUpload

import encodings
import logging
import re


logger = logging.getLogger('ATCT')

if HAS_MX_TIDY:
    from mx.Tidy import tidy as mx_tidy

# matches something like 'line 15 column 1 - Warning: missing ...'
RE_MATCH_WARNING = re.compile('^line (\d+) column (\d+) - Warning: (.*)$')
WARNING_LINE = 'line %d column %d - Warning: %s'

# matches something like 'line 15 column 1 - Error: missing ...'
RE_MATCH_ERROR = re.compile('^line (\d+) column (\d+) - Error: (.*)$')
ERROR_LINE = 'line %d column %d - Error: %s'

# The following regex is safe because *? matches the minimal text in the body
# tag and .* matches the maximum text between two body tags including other
# body tags if they exists
RE_BODY = re.compile('<body[^>]*?>(.*)</body>', re.DOTALL)

# get the encoding from an uploaded html-page
# e.g. <meta http-equiv="content-type" content="text/html; charset=ISO-8859-1">
# we get ISO-8859-1 into the second match, the rest into the first and third.
RE_GET_HTML_ENCODING = re.compile(
    '(<meta.*?content-type.*?charset[\s]*=[\s]*)([^"]*?)("[^>]*?>)',
    re.S | re.I)

# subtract 11 line numbers from the warning/error
SUBTRACT_LINES = 11

validatorList = []


class TALValidator:
    """Validates a text to be valid TAL code

    """
    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kw):
        gen = TALGenerator(getEngine(), xml=1, source_file=None)
        parser = HTMLTALParser(gen)
        try:
            parser.parseString(value)
        except Exception as err:
            return ("Validation Failed(%s): \n %s" % (self.name, err))
        return 1

validatorList.append(TALValidator('isTAL', title='', description=''))


class TidyHtmlValidator:
    """use mxTidy to check HTML

    Fail on errors and warnings
    Do not clean up the value
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kw):
        if not (HAS_MX_TIDY and MX_TIDY_ENABLED):
            # no mxTidy installed
            return 1

        request = kw['REQUEST']
        field = kw['field']

        result = doTidy(value, field, request)
        if result is None:
            return 1

        nerrors, nwarnings, outputdata, errordata = result
        errors = nerrors + nwarnings

        if errors:
            return ("Validation Failed(%s): \n %s" % (self.name, errordata))
        else:
            return 1

validatorList.append(TidyHtmlValidator('isTidyHtml', title='', description=''))


class TidyHtmlWithCleanupValidator:
    """use mxTidy to check HTML

    Fail only on errors
    Clean up
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kw):
        if not (HAS_MX_TIDY and MX_TIDY_ENABLED):
            # no mxTidy installed
            return 1

        request = kw['REQUEST']
        field = kw['field']

        result = doTidy(value, field, request, cleanup=1)
        if result is None:
            return 1

        nerrors, nwarnings, outputdata, errordata = result
        errors = nerrors

        # save the changed output in the request
        tidyAttribute = '%s_tidier_data' % field.getName()
        request[tidyAttribute] = outputdata

        if nwarnings:
            tidiedFields = list(request.get('tidiedFields', []))
            tidiedFields.append(field)
            request.set('tidiedFields', tidiedFields)

        if errors:
            return ("Validation Failed(%s): \n %s" % (self.name, errordata))
        else:
            return 1

validatorList.append(TidyHtmlWithCleanupValidator(
    'isTidyHtmlWithCleanup', title='', description=''))


class NonEmptyFileValidator:
    """Fails on empty non-existant files
    """

    implements(IValidator)

    def __init__(self, name, title='', description=''):
        self.name = name
        self.title = title or name
        self.description = description

    def __call__(self, value, *args, **kwargs):
        # calculate size
        if isinstance(value, FileUpload) or isinstance(value, FileType) \
                or hasattr(aq_base(value), 'tell'):
            value.seek(0, 2)  # eof
            size = value.tell()
            value.seek(0)
        else:
            try:
                size = len(value)
            except TypeError:
                size = 1

        if size == 0:
            return ("Validation failed: Uploaded file is empty")
        else:
            return True


validatorList.append(NonEmptyFileValidator(
    'isNonEmptyFile', title='', description=''))

for validator in validatorList:
    # register the validators
    validation.register(validator)


def doTidy(value, field, request, cleanup=0):
    """Tidy the data in 'value' for the field in the current request

    Optional cleanup:
      * removes header/footer of the output data
      * Removes warnings from the error data

    Return None for 'nothing done'
    else return (nerrors, nwarnings, outputdata, errordata)
    """
    # we can't use the mimetype from the field because it's updated *after*
    # validation so we must get it from the request
    tf_name = '%s_text_format' % field.getName()
    text_format = getattr(request, tf_name, '')

    # MX_TIDY_MIMETYPES configuration option isn't empty
    # and the current text_format isn't in the list
    if MX_TIDY_MIMETYPES and text_format not in MX_TIDY_MIMETYPES:
        # do not filter this mime type
        return

    # it's a file upload
    if isinstance(value, FileUpload):
        # *mmh* ok it's a file upload but a file upload could destroy
        # the layout, too.
        # the validator can be called many times, we have to rewind
        # the FileUpload.
        value.seek(0)
        value = correctEncoding(value.read())
    else:
        value = wrapValueInHTML(value)

    result = mx_tidy(value, **MX_TIDY_OPTIONS)
    nerrors, nwarnings, outputdata, errordata = result

    # parse and change the error data
    errordata = parseErrorData(errordata, removeWarnings=cleanup)
    if cleanup and outputdata:
        # unwrap tidied output data
        outputdata = unwrapValueFromHTML(outputdata)

    return nerrors, nwarnings, outputdata, errordata


def wrapValueInHTML(value):
    """Wrap the data in a valid html construct to remove the missing title error
    """
    return """
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head>
<title></title>
</head>
<body>
%s
</body>
</html>
""" % value


def unwrapValueFromHTML(value):
    """Remove the html stuff around the body
    """
    # get the body text
    result = RE_BODY.search(value)
    if result:
        body = result.group(1)
    else:
        raise ValueError('%s is not a html string' % value)
    return body


def correctEncoding(value):
    """correct the encoding of a html-page if we know it an mxTidy
       expects an other encoding
    """

    # we have nothing to do if mxTidy has no
    # fixed char_encoding
    if 'char_encoding' not in MX_TIDY_OPTIONS  \
            or (MX_TIDY_OPTIONS['char_encoding'] == 'raw'):
        return value

    match = RE_GET_HTML_ENCODING.search(value)
    if match:
        groups = match.groups()

        # lookup encodings in the pyhon encodings database
        # returns function-pointers that we can compare
        # need to normalize encodings a bit before
        html_encoding = groups[1].strip().lower()
        char_encoding = MX_TIDY_OPTIONS['char_encoding'].lower().strip()
        h_enc = encodings.search_function(html_encoding)
        c_enc = encodings.search_function(char_encoding)

        # one encoding is missing or they are equal
        if not (h_enc and c_enc) or h_enc == c_enc:
            return value
        else:
            try:
                return unicode(value, html_encoding).encode(char_encoding)
            except:
                logger.info("Error correcting encoding from %s to %s" %
                            (html_encoding, char_encoding))
    return value


def parseErrorData(data, removeWarnings=0):
    """Parse the error data to change some stuff
    """
    lines = data.split('\n')
    nlines = []
    for line in lines:
        # substract 11 lines from line
        error = RE_MATCH_ERROR.search(line)
        if error:
            # an error line
            lnum, cnum, text = error.groups()
            lnum = int(lnum) - SUBTRACT_LINES
            cnum = int(cnum)
            nlines.append(ERROR_LINE % (lnum, cnum, text))
        else:
            warning = RE_MATCH_WARNING.search(line)
            if warning and not removeWarnings:
                # a warning line and add warnings to output
                lnum, cnum, text = warning.groups()
                lnum = int(lnum) - SUBTRACT_LINES
                cnum = int(cnum)
                nlines.append(WARNING_LINE % (lnum, cnum, text))
            else:
                # something else
                nlines.append(line)
    return '\n'.join(nlines)



# -*- coding: utf-8 -*-
from plone.app.testing.bbb import PloneTestCase
from Products.CMFCore.utils import getToolByName


class TestBaseProfile(PloneTestCase):

    def afterSetUp(self):
        self.loginAsPortalOwner()
        qi = self.portal.portal_quickinstaller
        qi.uninstallProducts(['ATContentTypes'])
        portal_setup = self.portal.portal_setup
        portal_setup.runAllImportStepsFromProfile(
            'profile-Products.ATContentTypes:base')

    def test_attypes_not_installed(self):
        tt = getToolByName(self.portal, 'portal_types')
        types = tt.listTypeInfo()
        for t in types:
            self.assertNotEqual(t.product, 'ATContentTypes')



# -*- coding: utf-8 -*-
from UserDict import UserDict
from Products.Archetypes.atapi import registerType
from Products.ATContentTypes.config import PROJECTNAME
from types import StringType

from Products.ATContentTypes.interfaces import IATTopicCriterion
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.interfaces import IATTopicSortCriterion

ALL_INDICES = ('DateIndex', 'DateRangeIndex', 'FieldIndex', 'KeywordIndex',
               'PathIndex', 'TextIndex', 'TextIndexNG2', 'TextIndexNG3',
               'TopicIndex', 'ZCTextIndex', 'NavtreeIndexNG',
               'ExtendedPathIndex', 'BooleanIndex', 'UUIDIndex')

SORT_INDICES = ('DateIndex', 'DateRangeIndex', 'FieldIndex', 'KeywordIndex',
                'PathIndex', 'GopipIndex')
# TextIndex, TopicIndex, ZCTextIndex, TextIndexNG2, NavtreeIndexNG
# are not usable to sort as they do not have 'keyForDocument' attribute

DATE_INDICES = ('DateIndex', 'DateRangeIndex', 'FieldIndex')

# Indices that can take a list of values
LIST_INDICES = ('FieldIndex', 'KeywordIndex', 'PathIndex', 'NavtreeIndexNG',
                'ExtendedPathIndex', 'TopicIndex')

TEXT_INDICES = ('TextIndex', 'TextIndexNG2', 'TextIndexNG3', 'ZCTextIndex')

# Indices that can take a simple string
STRING_INDICES = LIST_INDICES + TEXT_INDICES

# Indices that may hold AT reference data
REFERENCE_INDICES = ('FieldIndex', 'KeywordIndex')
FIELD_INDICES = ('FieldIndex',)
PATH_INDICES = ('PathIndex', 'ExtendedPathIndex')


class _CriterionRegistry(UserDict):
    """Registry for criteria """

    def __init__(self, *args, **kwargs):
        UserDict.__init__(self, *args, **kwargs)
        self.index2criterion = {}
        self.criterion2index = {}
        self.portaltypes = {}

    def register(self, criterion, indices):
        if isinstance(indices, StringType):
            indices = (indices,)
        indices = tuple(indices)

        if indices == ():
            indices = ALL_INDICES

        assert IATTopicCriterion.implementedBy(criterion)
        registerType(criterion, PROJECTNAME)

        crit_id = criterion.meta_type
        self[crit_id] = criterion
        self.portaltypes[criterion.portal_type] = criterion

        self.criterion2index[crit_id] = indices
        for index in indices:
            value = self.index2criterion.get(index, ())
            self.index2criterion[index] = value + (crit_id,)

    def unregister(self, criterion):
        crit_id = criterion.meta_type
        self.pop(crit_id)
        self.criterion2index.pop(crit_id)
        for (index, value) in self.index2criterion.items():
            if id in value:
                valuelist = list(value)
                del valuelist[valuelist.index(crit_id)]
                self.index2criterion[index] = tuple(valuelist)

    def listTypes(self):
        return self.keys()

    def listSortTypes(self):
        return [key for key in self.keys()
                if IATTopicSortCriterion.implementedBy(self[key])]

    def listSearchTypes(self):
        return [key for key in self.keys()
                if IATTopicSearchCriterion.implementedBy(self[key])]

    def listCriteria(self):
        return self.values()

    def indicesByCriterion(self, criterion):
        return self.criterion2index[criterion]

    def criteriaByIndex(self, index):
        try:
            return self.index2criterion[index]
        except KeyError:
            return ()

    def getPortalTypes(self):
        return tuple(self.portaltypes.keys())

_criterionRegistry = _CriterionRegistry()
registerCriterion = _criterionRegistry.register
unregisterCriterion = _criterionRegistry.unregister

__all__ = (
    'registerCriterion',
    'ALL_INDICES',
    'DATE_INDICES',
    'STRING_INDICES',
    'LIST_INDICES',
    'SORT_INDICES',
)

# criteria
from Products.ATContentTypes.criteria.boolean import ATBooleanCriterion  # noqa
from Products.ATContentTypes.criteria.date import ATDateCriteria  # noqa
from Products.ATContentTypes.criteria.daterange import ATDateRangeCriterion  # noqa
from Products.ATContentTypes.criteria.list import ATListCriterion  # noqa
from Products.ATContentTypes.criteria.portaltype import ATPortalTypeCriterion  # noqa
from Products.ATContentTypes.criteria.reference import ATReferenceCriterion  # noqa
from Products.ATContentTypes.criteria.selection import ATSelectionCriterion  # noqa
from Products.ATContentTypes.criteria.simpleint import ATSimpleIntCriterion  # noqa
from Products.ATContentTypes.criteria.simplestring import ATSimpleStringCriterion  # noqa
from Products.ATContentTypes.criteria.sort import ATSortCriterion  # noqa
from Products.ATContentTypes.criteria.currentauthor import ATCurrentAuthorCriterion  # noqa
from Products.ATContentTypes.criteria.path import ATPathCriterion  # noqa
from Products.ATContentTypes.criteria.relativepath import ATRelativePathCriterion  # noqa



# -*- coding: utf-8 -*-

from plone.app.testing.bbb import PTC_FUNCTIONAL_TESTING
from plone.testing import layered

import doctest

FILES = [
    'events.txt',
    'editing.txt',
    'topictool.txt',
    'portaltype_criterion.txt',
    'webdav.txt',
    'http_access.txt',
    'reindex_sanity.txt',
    'uploading.txt',
    'browser_collection_views.txt',
    # traversal.txt registers the browser page "document_view", and this
    # registration stays active in different doctests, so we make sure to
    # include it last.
    'traversal.txt',
]

OPTIONFLAGS = (doctest.ELLIPSIS |
               doctest.NORMALIZE_WHITESPACE |
               doctest.REPORT_NDIFF |
               doctest.REPORT_ONLY_FIRST_FAILURE)


def test_suite():
    import unittest
    suite = unittest.TestSuite()
    for testfile in FILES:
        suite.addTest(layered(
            doctest.DocFileSuite(
                testfile,
                optionflags=OPTIONFLAGS,
                package="Products.ATContentTypes.tests", ),
            layer=PTC_FUNCTIONAL_TESTING)
        )
    return suite



# -*- coding: utf-8 -*-
from Products.ATContentTypes.interfaces.document import IATDocument  # noqa
from Products.ATContentTypes.interfaces.event import IATEvent  # noqa
from Products.ATContentTypes.interfaces.factory import IFactoryTool  # noqa
from Products.ATContentTypes.interfaces.file import IATFile  # noqa
from Products.ATContentTypes.interfaces.file import IFileContent  # noqa
from Products.ATContentTypes.interfaces.folder import IATBTreeFolder  # noqa
from Products.ATContentTypes.interfaces.folder import IATFolder  # noqa
from Products.ATContentTypes.interfaces.folder import IFilterFolder  # noqa
from Products.ATContentTypes.interfaces.image import IATImage  # noqa
from Products.ATContentTypes.interfaces.image import IImageContent  # noqa
from Products.ATContentTypes.interfaces.interfaces import IATCTTool  # noqa
from Products.ATContentTypes.interfaces.interfaces import IATContentType  # noqa
from Products.ATContentTypes.interfaces.interfaces import ICalendarSupport  # noqa
from Products.ATContentTypes.interfaces.interfaces import IHistoryAware  # noqa
from Products.ATContentTypes.interfaces.interfaces import ISelectableConstrainTypes  # noqa
from Products.ATContentTypes.interfaces.interfaces import ITextContent  # noqa
from Products.ATContentTypes.interfaces.link import IATLink  # noqa
from Products.ATContentTypes.interfaces.news import IATNewsItem  # noqa
from Products.ATContentTypes.interfaces.topic import IATCTTopicsTool  # noqa
from Products.ATContentTypes.interfaces.topic import IATTopic  # noqa
from Products.ATContentTypes.interfaces.topic import IATTopicCriterion  # noqa
from Products.ATContentTypes.interfaces.topic import IATTopicSearchCriterion  # noqa
from Products.ATContentTypes.interfaces.topic import IATTopicSortCriterion  # noqa
from Products.ATContentTypes.interfaces.factories import IATCTFileFactory  # noqa



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import DisplayList
from Products.Archetypes.atapi import IntegerField
from Products.Archetypes.atapi import IntegerWidget
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import SelectionWidget
from Products.Archetypes.atapi import StringField
from Products.ATContentTypes import ATCTMessageFactory as _
from Products.ATContentTypes.criteria import LIST_INDICES
from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics
from Products.CMFCore.permissions import View
from zope.interface import implements


DirectionOperations = DisplayList((
    ('', _(u'Equal to')),
    ('min', _(u'Greater than')),
    ('max', _(u'Less than')),
    ('min:max', _(u'Between')),
))

ATSimpleIntCriterionSchema = ATBaseCriterionSchema + Schema((
    IntegerField(
        'value',
        required=1,
        mode="rw",
        write_permission=ChangeTopics,
        accessor="Value",
        mutator="setValue",
        default=None,
        widget=IntegerWidget(
            label=_(u'label_int_criteria_value', default=u'Value'),
            description=_(u'help_int_criteria_value',
                          default=u'An integer number.')
        ),
    ),
    IntegerField(
        'value2',
        required=0,
        mode="rw",
        write_permission=ChangeTopics,
        accessor="Value2",
        mutator="setValue2",
        default=None,
        widget=IntegerWidget(
            label=_(u'label_int_criteria_value2',
                    default=u'Second Value'),
            description=_(
                u'help_int_criteria_value2',
                default=u'An integer number used as the maximum value if '
                u'the between direction is selected.')
        ),
    ),
    StringField(
        'direction',
        required=0,
        mode="rw",
        write_permission=ChangeTopics,
        default='',
        vocabulary=DirectionOperations,
        enforceVocabulary=1,
        widget=SelectionWidget(
            label=_(u'label_int_criteria_direction',
                    default=u'Direction'),
            description=_(
                u'help_int_criteria_direction',
                default=u'Specify whether you want to find values lesser '
                u'than, greater than, equal to, or between the chosen '
                u'value(s).')
        ),
    ),
))


class ATSimpleIntCriterion(ATBaseCriterion):
    """A simple int criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATSimpleIntCriterionSchema
    meta_type = 'ATSimpleIntCriterion'
    archetype_name = 'Simple Int Criterion'
    shortDesc = 'Integer value or range'

    @security.protected(View)
    def getCriteriaItems(self):
        result = []
        val = self.Value()
        direction = self.getDirection()
        if val or val == 0:
            if direction == 'min:max':
                val = tuple([int(val), int(self.Value2())])
            else:
                val = int(val)
            if direction:
                result.append(
                    (self.Field(), {'query': val, 'range': direction}))
            else:
                result.append((self.Field(), {'query': val}))

        return tuple(result)

    @security.protected(View)
    def post_validate(self, REQUEST, errors):
        """Check that Value2 is set if range is set to min:max"""
        direction = REQUEST.get('direction', self.getDirection())
        val2 = REQUEST.get('value2', self.Value2())
        if direction == 'min:max' and not val2 and not val2 == 0:
            errors['value2'] = (
                'You must enter a second value to do a "Between" search.')
        return errors

registerCriterion(ATSimpleIntCriterion, LIST_INDICES)



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from Missing import MV
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import BooleanWidget
from Products.Archetypes.atapi import Schema
from Products.ATContentTypes import ATCTMessageFactory as _
from Products.ATContentTypes.criteria import FIELD_INDICES
from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics
from Products.CMFCore.permissions import View
from zope.interface import implements


ATBooleanCriterionSchema = ATBaseCriterionSchema + Schema((
    BooleanField('bool',
                 required=1,
                 mode="rw",
                 write_permission=ChangeTopics,
                 default=None,
                 widget=BooleanWidget(
                     label=_(u'label_boolean_criteria_bool', default=u'Value'),
                     description=_(u'help_boolean_criteria_bool',
                                   default=u'True or false')
                 ),
                 ),
))


class ATBooleanCriterion(ATBaseCriterion):
    """A boolean criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATBooleanCriterionSchema
    meta_type = 'ATBooleanCriterion'
    archetype_name = 'Boolean Criterion'
    shortDesc = 'Boolean (True/False)'

    @security.protected(View)
    def getCriteriaItems(self):
        result = []
        if self.getBool():
            value = [1, True, '1', 'True']
        else:
            value = [0, '', False, '0', 'False', None, (), [], {}, MV]
        result.append((self.Field(), value))

        return tuple(result)

registerCriterion(ATBooleanCriterion, FIELD_INDICES + ('BooleanIndex', ))



# -*- coding: utf-8 -*-
from Products.ATContentTypes.interfaces import IFactoryTool
from Products.CMFCore.utils import getToolByName
from Products.GenericSetup.utils import exportObjects
from Products.GenericSetup.utils import importObjects
from Products.GenericSetup.utils import XMLAdapterBase


class PortalFactoryXMLAdapter(XMLAdapterBase):
    """In- and exporter for FactoryTool.
    """

    __used_for__ = IFactoryTool

    _LOGGER_ID = name = 'factorytool'

    def _exportNode(self):
        """Export the object as a DOM node.
        """
        node = self._getObjectNode("object")
        node.appendChild(self._extractFactoryToolSettings())

        self._logger.info("FactoryTool settings exported.")
        return node

    def _importNode(self, node):
        if self.environ.shouldPurge():
            self._purgeFactoryToolSettings()

        self._initFactoryToolSettings(node)
        self._logger.info("FactoryTool settings imported.")

    def _purgeFactoryToolSettings(self):
        self.context.manage_setPortalFactoryTypes(listOfTypeIds=[])

    def _initFactoryToolSettings(self, node):
        for child in node.childNodes:
            if child.nodeName == "factorytypes":
                types = set(self.context.getFactoryTypes())
                for type in child.getElementsByTagName("type"):
                    types.add(type.getAttribute("portal_type"))
                self.context.manage_setPortalFactoryTypes(
                    listOfTypeIds=list(types))

    def _extractFactoryToolSettings(self):
        node = self._doc.createElement("factorytypes")
        for t in sorted(self.context.getFactoryTypes()):
            child = self._doc.createElement("type")
            child.setAttribute("portal_type", t)
            node.appendChild(child)

        return node


def importFactoryTool(context):
    """Import Factory Tool configuration.
    """
    site = context.getSite()
    tool = getToolByName(site, 'portal_factory', None)
    if tool is None:
        return

    importObjects(tool, '', context)


def exportFactoryTool(context):
    """Export Factory Tool configuration.
    """
    site = context.getSite()
    tool = getToolByName(site, 'portal_factory', None)
    if tool is None:
        return

    exportObjects(tool, '', context)



# -*- coding: utf-8 -*-
from hashlib import sha1 as sha
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.keyring.interfaces import IKeyManager
from Products.ATContentTypes.config import HAS_LINGUA_PLONE
from Products.ATContentTypes.tests import atcttestcase
from zope.component import getUtility

import hmac


class IntegrationTestCase(atcttestcase.ATCTFunctionalSiteTestCase):

    views = ()

    def afterSetUp(self):
        super(IntegrationTestCase, self).afterSetUp()

        # basic data
        self.folder_url = self.folder.absolute_url()
        self.folder_path = '/%s' % self.folder.absolute_url(1)
        self.basic_auth = '%s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD)
        self.owner_auth = '%s:%s' % (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        # object
        self.setupTestObject()

    def setupTestObject(self):
        raise NotImplementedError

    def getAuthToken(self, user=TEST_USER_ID):
        manager = getUtility(IKeyManager)
        try:
            ring = manager[u"_forms"]
        except KeyError:
            ring = manager[u'_system']

        secret = ring.random()
        return hmac.new(secret, user, sha).hexdigest()


class ATCTIntegrationTestCase(IntegrationTestCase):
    """Integration tests for view and edit templates
    """

    portal_type = None

    def beforeTearDown(self):
        del self.folder[self.obj_id]

    def setupTestObject(self):
        # create test object
        self.obj_id = 'test_object'
        self.title = u'test \xf6bject'
        if self.obj_id not in self.folder:
            self.folder.invokeFactory(
                self.portal_type, self.obj_id, title=self.title)
        self.obj = getattr(self.folder.aq_explicit, self.obj_id)
        self.obj_url = self.obj.absolute_url()
        self.obj_path = '/%s' % self.obj.absolute_url(1)

    def test_createObject(self):
        # create an object using the createObject script
        auth = self.getAuthToken()
        response = self.publish(
            '%s/createObject?type_name=%s&_authenticator=%s' % (
                self.folder_path, self.portal_type, auth),
            self.basic_auth)

        self.assertEqual(response.getStatus(), 302)  # Redirect to edit

        body = response.getBody()

        self.assertTrue(body.startswith(self.folder_url), body)
        # The url may end with /edit or /atct_edit depending on method aliases
        self.assertTrue('edit' in body, body)

        # Perform the redirect
        edit_form_path = body[len(self.layer['request'].SERVER_URL):]
        response = self.publish(edit_form_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK
        temp_id = body.split('/')[-2]

        new_obj = self.folder.portal_factory._getTempFolder(self.portal_type)[
            temp_id]
        # object is not yet edited
        self.assertEqual(new_obj.checkCreationFlag(), True)

    def check_newly_created(self):
        """Objects created programmatically should not have creation flag set.
        """
        self.assertEqual(self.obj.checkCreationFlag(),
                         False)  # object is fully created

    def test_edit_view(self):
        # edit should work
        response = self.publish(
            '%s/atct_edit?_authenticator=%s' % (
                self.obj_path, self.getAuthToken()),
            self.basic_auth)
        self.assertTrue(response.getBody().startswith('<!DOCTYPE html'))
        self.assertTrue(response.getStatus() in (302, 200))  # OK

    def test_base_view(self):
        # base view should work
        response = self.publish('%s/base_view' %
                                self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def test_dynamic_view(self):
        # dynamic view magic should work
        response = self.publish('%s/view' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def test_local_sharing_view(self):
        # sharing tab should work
        response = self.publish('%s/sharing' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    # LinguaPlone specific tests
    if HAS_LINGUA_PLONE:

        def test_linguaplone_views(self):
            response = self.publish('%s/translate_item' %
                                    self.obj_path, self.basic_auth)
            self.assertEqual(response.getStatus(), 200)  # OK
            response = self.publish(
                '%s/manage_translations_form' % self.obj_path, self.basic_auth)
            self.assertEqual(response.getStatus(), 200)  # OK

        def test_linguaplone_create_translation(self):
            # create translation creates a new object
            response = self.publish(
                '{}/createTranslation?language=de&set_language=de'.format(
                    self.obj_path), self.basic_auth)
            self.assertEqual(response.getStatus(), 302)  # Redirect

            body = response.getBody()
            self.assertTrue(body.startswith(self.folder_url))
            self.assertTrue(body.endswith('/translate_item'))

            # Perform the redirect
            form_path = body[len(self.app.REQUEST.SERVER_URL):]
            response = self.publish(form_path, self.basic_auth)
            self.assertEqual(response.getStatus(), 200)  # OK

            translated_id = "%s-de" % self.obj_id
            self.assertTrue(translated_id in self.folder,
                            self.folder)

    def test_additional_view(self):
        # additional views:
        for view in self.views:
            response = self.publish(
                '%s/%s' % (self.obj_path, view), self.basic_auth)
            self.assertEqual(response.getStatus(), 200,
                             "%s: %s" % (view, response.getStatus()))  # OK

    def test_dynamicViewContext(self):
        # register and add a testing template (it's a script)
        self.setRoles(['Manager', 'Member'])

        ttool = self.portal.portal_types
        fti = getattr(ttool, self.portal_type)
        view_methods = fti.getAvailableViewMethods(
            self.obj) + ('unittestGetTitleOf',)
        fti.manage_changeProperties(view_methods=view_methods)

        self.obj.setLayout('unittestGetTitleOf')
        self.folder.setTitle('the folder')
        self.obj.setTitle('the obj')

        self.setRoles(['Member'])

        response = self.publish('%s/view' % self.obj_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

        output = response.getBody().split(',')
        self.assertEqual(len(output), 4, output)

        self.assertEqual(
            output, ['the obj', 'the folder', 'the obj', 'the folder'])



# -*- coding: utf-8 -*-
##############################################################################
#
# Copyright (c) 2001 Zope Foundation and Contributors.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
""" ATContentTypes portal_metadata tool. """

from AccessControl.SecurityInfo import ClassSecurityInfo
from App.class_init import InitializeClass
from App.special_dtml import DTMLFile
from OFS.Folder import Folder
from OFS.SimpleItem import SimpleItem
from Persistence import PersistentMapping
from Products.ATContentTypes.config import WWW_DIR
from Products.CMFCore.interfaces import IMetadataTool
from Products.CMFCore.utils import registerToolInterface
from Products.CMFCore.utils import UniqueObject
from Products.CMFPlone.permissions import ManagePortal
from Products.CMFPlone.permissions import ModifyPortalContent
from Products.CMFPlone.permissions import View
from Products.CMFPlone.PloneBaseTool import PloneBaseTool
from zope.interface import implements


class MetadataError(Exception):
    """ Metadata error.  """


class MetadataElementPolicy(SimpleItem):

    """ Represent a type-specific policy about a particular metadata element.
    """

    security = ClassSecurityInfo()
    #
    #   Default values.
    #
    is_required = 0
    supply_default = 0
    default_value = ''
    enforce_vocabulary = 0
    allowed_vocabulary = ()

    def __init__(self, is_multi_valued=False):
        self.is_multi_valued = bool(is_multi_valued)

    #
    #   Mutator.
    #
    @security.protected(ManagePortal)
    def edit(self, is_required, supply_default, default_value,
             enforce_vocabulary, allowed_vocabulary):
        self.is_required = bool(is_required)
        self.supply_default = bool(supply_default)
        self.default_value = default_value
        self.enforce_vocabulary = bool(enforce_vocabulary)
        self.allowed_vocabulary = tuple(allowed_vocabulary)

    #
    #   Query interface
    #
    @security.protected(View)
    def isMultiValued(self):
        """ Can this element hold multiple values?
        """
        return self.is_multi_valued

    @security.protected(View)
    def isRequired(self):
        """ Must this element be supplied?
        """
        return self.is_required

    @security.protected(View)
    def supplyDefault(self):
        """ Should the tool supply a default?
        """
        return self.supply_default

    @security.protected(View)
    def defaultValue(self):
        """ If so, what is the default?
        """
        return self.default_value

    @security.protected(View)
    def enforceVocabulary(self):
        """ Should the tool enforce the policy's vocabulary?
        """
        return self.enforce_vocabulary

    @security.protected(View)
    def allowedVocabulary(self):
        """ What are the allowed values?
        """
        return self.allowed_vocabulary

InitializeClass(MetadataElementPolicy)


class ElementSpec(SimpleItem):

    """ Represent all the tool knows about a single metadata element.
    """

    security = ClassSecurityInfo()

    #
    #   Default values.
    #
    is_multi_valued = 0

    def __init__(self, is_multi_valued):
        self.is_multi_valued = is_multi_valued
        self.policies = PersistentMapping()
        self.policies[None] = self._makePolicy()  # set default policy

    @security.private
    def _makePolicy(self):
        return MetadataElementPolicy(self.is_multi_valued)

    @security.protected(View)
    def isMultiValued(self):
        """
            Is this element multi-valued?
        """
        return self.is_multi_valued

    @security.protected(View)
    def getPolicy(self, typ=None):
        """ Find the policy for this element for objects of the given type.

        o Return a default, if none found.
        """
        try:
            return self.policies[typ].__of__(self)
        except KeyError:
            return self.policies[None].__of__(self)

    @security.protected(View)
    def listPolicies(self):
        """ Return a list of all policies for this element.
        """
        res = []
        for k, v in self.policies.items():
            res.append((k, v.__of__(self)))
        return res

    @security.protected(ManagePortal)
    def addPolicy(self, typ):
        """ Add a policy to this element for objects of the given type.
        """
        if typ is None:
            raise MetadataError("Can't replace default policy.")

        if typ in self.policies:
            raise MetadataError("Existing policy for content type:" + typ)

        self.policies[typ] = self._makePolicy()

    @security.protected(ManagePortal)
    def removePolicy(self, typ):
        """ Remove the policy from this element for objects of the given type.

        o Do *not* remvoe the default, however.
        """
        if typ is None:
            raise MetadataError("Can't remove default policy.")
        del self.policies[typ]

InitializeClass(ElementSpec)


class MetadataSchema(SimpleItem):

    """ Describe a metadata schema.
    """

    security = ClassSecurityInfo()

    meta_type = 'Metadata Schema'
    publisher = ''

    def __init__(self, id, element_specs=()):
        self._setId(id)
        self.element_specs = PersistentMapping()
        for name, is_multi_valued in element_specs:
            self.element_specs[name] = ElementSpec(is_multi_valued)

    #
    #   ZMI methods
    #
    manage_options = (
        ({'label': 'Elements', 'action': 'elementPoliciesForm'}, ) +
        SimpleItem.manage_options)

    security.declareProtected(ManagePortal, 'elementPoliciesForm')
    elementPoliciesForm = DTMLFile('metadataElementPolicies', WWW_DIR)

    @security.protected(ManagePortal)
    def addElementPolicy(self, element, content_type, is_required,
                         supply_default, default_value, enforce_vocabulary,
                         allowed_vocabulary, REQUEST=None):
        """ Add a type-specific policy for one of our elements.
        """
        if content_type == '<default>':
            content_type = None

        spec = self.getElementSpec(element)
        spec.addPolicy(content_type)
        policy = spec.getPolicy(content_type)
        policy.edit(is_required, supply_default, default_value,
                    enforce_vocabulary, allowed_vocabulary)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url() +
                '/elementPoliciesForm?element=' + element +
                '&manage_tabs_message=Policy+added.')

    @security.protected(ManagePortal)
    def removeElementPolicy(self, element, content_type, REQUEST=None
                            ):
        """ Remvoe a type-specific policy for one of our elements.
        """
        if content_type == '<default>':
            content_type = None

        spec = self.getElementSpec(element)
        spec.removePolicy(content_type)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url() +
                '/elementPoliciesForm?element=' + element +
                '&manage_tabs_message=Policy+removed.')

    @security.protected(ManagePortal)
    def updateElementPolicy(self, element, content_type, is_required,
                            supply_default, default_value, enforce_vocabulary,
                            allowed_vocabulary, REQUEST=None):
        """ Update a policy for one of our elements

        o 'content_type' will be '<default>' when we edit the default.
        """
        if content_type == '<default>':
            content_type = None
        spec = self.getElementSpec(element)
        policy = spec.getPolicy(content_type)
        policy.edit(is_required, supply_default, default_value,
                    enforce_vocabulary, allowed_vocabulary)
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url() +
                '/elementPoliciesForm?element=' + element +
                '&manage_tabs_message=Policy+updated.')

    #
    #   Element spec manipulation.
    #
    @security.protected(ManagePortal)
    def listElementSpecs(self):
        """ Return a list of ElementSpecs representing the elements we manage.
        """
        res = []
        for k, v in self.element_specs.items():
            res.append((k, v.__of__(self)))
        return res

    @security.protected(ManagePortal)
    def getElementSpec(self, element):
        """ Return an ElementSpec for the given 'element'.
        """
        return self.element_specs[element].__of__(self)

    @security.protected(ManagePortal)
    def addElementSpec(self, element, is_multi_valued, REQUEST=None):
        """ Add 'element' to our list of managed elements.
        """
        # Don't replace.
        if element in self.element_specs:
            return

        self.element_specs[element] = ElementSpec(is_multi_valued)

        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url() +
                '/propertiesForm?manage_tabs_message=Element+' + element +
                '+added.')

    @security.protected(ManagePortal)
    def removeElementSpec(self, element, REQUEST=None):
        """ Remove 'element' from our list of managed elements.
        """
        del self.element_specs[element]

        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url() +
                '/propertiesForm?manage_tabs_message=Element+' + element +
                '+removed.')

    @security.protected(ManagePortal)
    def listPolicies(self, typ=None):
        """ Show all policies for a given content type

        o If 'typ' is none, return the list of default policies.
        """
        result = []
        for element, spec in self.listElementSpecs():
            result.append((element, spec.getPolicy(typ)))
        return result

InitializeClass(MetadataSchema)


_DCMI_ELEMENT_SPECS = (
    ('Title', 0),
    ('Description', 0),
    ('Subject', 1),
    ('Format', 0),
    ('Language', 0),
    ('Rights', 0)
)


class MetadataTool(PloneBaseTool, UniqueObject, Folder):

    implements(IMetadataTool)

    id = 'portal_metadata'
    meta_type = 'Default Metadata Tool'
    toolicon = 'skins/plone_images/info_icon.png'

    #
    #   Default values.
    #
    publisher = ''

    security = ClassSecurityInfo()

    def __init__(self, publisher=None):

        self.editProperties(publisher)
        self.DCMI = MetadataSchema('DCMI', _DCMI_ELEMENT_SPECS)

    #
    #   ZMI methods
    #
    manage_options = (
        ({'label': 'Schema', 'action': 'propertiesForm'},
         {'label': 'Overview', 'action': 'manage_overview'}) +
        Folder.manage_options)

    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('explainMetadataTool', WWW_DIR)

    security.declareProtected(ManagePortal, 'propertiesForm')
    propertiesForm = DTMLFile('metadataProperties', WWW_DIR)

    @security.protected(ManagePortal)
    def editProperties(self, publisher=None, REQUEST=None):
        """ Form handler for "tool-wide" properties
        """
        if publisher is not None:
            self.publisher = publisher

        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(
                self.absolute_url() +
                '/propertiesForm?manage_tabs_message=Tool+updated.')

    @security.protected(ManagePortal)
    def manage_addSchema(self, schema_id, elements, REQUEST):
        """ ZMI wrapper around addSchema
        """
        massaged = []
        for element in elements:
            if isinstance(element, basestring):
                element = element.split(',')
                if len(element) < 2:
                    element.append(0)
            massaged.append(element)
        self.addSchema(schema_id, massaged)

        REQUEST['RESPONSE'].redirect(
            self.absolute_url() +
            '/propertiesForm?manage_tabs_message=Schema+added.')

    @security.protected(ManagePortal)
    def manage_removeSchemas(self, schema_ids, REQUEST):
        """ ZMI wrapper around removeSchema
        """
        if not schema_ids:
            raise ValueError('No schemas selected!')

        for schema_id in schema_ids:
            self.removeSchema(schema_id)

        REQUEST['RESPONSE'].redirect(
            self.absolute_url() +
            '/propertiesForm?manage_tabs_message=Schemas+removed.')

    @security.private
    def getFullName(self, userid):
        """ See IMetadataTool.
        """
        return userid   # TODO: do lookup here

    @security.public
    def getPublisher(self):
        """ See IMetadataTool.
        """
        return self.publisher

    @security.public
    def listAllowedSubjects(self, content=None, content_type=None):
        """ See IMetadataTool.
        """
        return self.listAllowedVocabulary(
            'DCMI', 'Subject', content, content_type)

    @security.public
    def listAllowedFormats(self, content=None, content_type=None):
        """ See IMetadataTool.
        """
        return self.listAllowedVocabulary(
            'DCMI', 'Format', content, content_type)

    @security.public
    def listAllowedLanguages(self, content=None, content_type=None):
        """ See IMetadataTool.
        """
        return self.listAllowedVocabulary(
            'DCMI', 'Language', content, content_type)

    @security.public
    def listAllowedRights(self, content=None, content_type=None):
        """ See IMetadata Tool.
        """
        return self.listAllowedVocabulary(
            'DCMI', 'Rights', content, content_type)

    @security.public
    def listAllowedVocabulary(self, schema, element, content=None,
                              content_type=None):
        """ See IMetadataTool.
        """
        schema_def = getattr(self, schema)
        spec = schema_def.getElementSpec(element)
        if content_type is None and content:
            content_type = content.getPortalTypeName()
        return spec.getPolicy(content_type).allowedVocabulary()

    @security.public
    def listSchemas(self):
        """ See IMetadataTool.
        """
        result = [('DCMI', self.DCMI)]
        result.extend(self.objectItems([MetadataSchema.meta_type]))
        return result

    @security.protected(ModifyPortalContent)
    def addSchema(self, schema_id, elements=()):
        """ See IMetadataTool.
        """
        if schema_id == 'DCMI' or schema_id in self.objectIds():
            raise KeyError('Duplicate schema ID: %s' % schema_id)

        schema = MetadataSchema(schema_id, elements)
        self._setObject(schema_id, schema)

        return self._getOb(schema_id)

    @security.protected(ModifyPortalContent)
    def removeSchema(self, schema_id):
        """ See IMetadataTool.
        """
        if schema_id == 'DCMI' or schema_id not in self.objectIds():
            raise KeyError('Invalid schema ID: %s' % schema_id)

        self._delObject(schema_id)

    @security.protected(ModifyPortalContent)
    def setInitialMetadata(self, content):
        """ See IMetadataTool.
        """
        for schema_id, schema in self.listSchemas():
            for element, policy in schema.listPolicies(
                    content.getPortalTypeName()):

                if not getattr(content, element)():

                    if policy.supplyDefault():
                        setter = getattr(content, 'set%s' % element)
                        setter(policy.defaultValue())
                    elif policy.isRequired():
                        raise MetadataError(
                            'Metadata element %s is required.' % element)

        # TODO:  Call initial_values_hook, if present

    @security.protected(View)
    def validateMetadata(self, content):
        """ See IMetadataTool.
        """
        for schema_id, schema in self.listSchemas():
            for element, policy in schema.listPolicies(
                    content.getPortalTypeName()):

                value = getattr(content, element)()
                if not value and policy.isRequired():
                    raise MetadataError(
                        'Metadata element %s is required.' %
                        element)

                if value and policy.enforceVocabulary():
                    values = policy.isMultiValued() and value or [value]
                    for value in values:
                        if value not in policy.allowedVocabulary():
                            raise MetadataError(
                                'Value %s is not in allowed vocabulary for '
                                'metadata element %s.' % (value, element))

InitializeClass(MetadataTool)
registerToolInterface('portal_metadata', IMetadataTool)



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import BooleanWidget
from Products.Archetypes.atapi import Schema
from Products.ATContentTypes import ATCTMessageFactory as _
from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria import SORT_INDICES
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSortCriterion
from Products.ATContentTypes.permission import ChangeTopics
from Products.CMFCore.permissions import View
from zope.interface import implements


ATSortCriterionSchema = ATBaseCriterionSchema + Schema((
    BooleanField('reversed',
                 required=0,
                 mode="rw",
                 write_permission=ChangeTopics,
                 default=0,
                 widget=BooleanWidget(label=_(u'Reverse')),
                 ),

))


class ATSortCriterion(ATBaseCriterion):
    """A sort criterion"""

    implements(IATTopicSortCriterion)
    security = ClassSecurityInfo()
    schema = ATSortCriterionSchema
    meta_type = 'ATSortCriterion'
    archetype_name = 'Sort Criterion'
    shortDesc = 'Sort'

    @security.protected(View)
    def getCriteriaItems(self):
        result = [('sort_on', self.Field())]

        if self.getReversed():
            result.append(('sort_order', 'reverse'))

        return tuple(result)

registerCriterion(ATSortCriterion, SORT_INDICES)



# -*- coding: utf-8 -*-
from Products.Archetypes.interfaces import IATHistoryAware
from Products.Archetypes.interfaces import IBaseContent
# Keep import for backwards compatibility:
from Products.Archetypes.interfaces import IBaseFolder  # noqa
from Products.CMFDynamicViewFTI.interfaces import ISelectableBrowserDefault
# Keep import for backwards compatibility:
from Products.CMFPlone.interfaces import ISelectableConstrainTypes  # noqa
from zope.interface import Attribute
from zope.interface import Interface


class IATContentType(ISelectableBrowserDefault, IBaseContent):
    """Marker interface for AT Content Types
    """

    default_view = Attribute(
        '''Default view template - used for dynamic view''')
    suppl_views = Attribute('''Supplementary views - used for dynamic view''')

    _atct_newTypeFor = Attribute('''XXX''')

    assocMimetypes = Attribute('''A tuple of mimetypes that are associated
                                  with this type. Format: ('bar/foo', 'foo/*',)
                               ''')

    assocFileExt = Attribute('''A tuple of file extensions that are associated
                                with this type. Format: ('jpeg', 'png',)
                             ''')

    cmf_edit_kws = Attribute('''List of keyword names.

    If one of this kw names is used with edit() then the cmf_edit method is
    called.
    ''')


class IHistoryAware(IATHistoryAware):
    """History awareness marker interface
    """

    def getHistorySource():
        """get source for HistoryAwareMixin

        Must return a (raw) string
        """

    def getLastEditor():
        """Returns the user name of the last editor.

        Returns None if no last editor is known.
        """

    def getDocumentComparisons(max=10, filterComment=0):
        """Get history as unified diff
        """


class ICalendarSupport(Interface):
    """Calendar import/export
    """


class ITextContent(Interface):
    """Interface for types containing text
    """

    def getText(**kwargs):
        """
        """

    def setText(value, **kwargs):
        """
        """

    def CookedBody(stx_level='ignored'):
        """
        """

    def EditableBody():
        """
        """


class IATCTTool(Interface):
    """
    """



# -*- coding: utf-8 -*-
from OFS.Folder import Folder
from Products.CMFPlone.exportimport.tests.base import BodyAdapterTestCase
from zope.component import provideAdapter
from zope.component import provideUtility


_FACTORYTOOL_XML = """\
<?xml version="1.0"?>
<object name="portal_factory" meta_type="Plone Factory Tool">
 <factorytypes>
  <type portal_type="Document"/>
  <type portal_type="Folder"/>
 </factorytypes>
</object>
"""


class DummyTypesTool(Folder):

    id = 'portal_types'
    meta_type = 'Dummy Types Tool'

    def listContentTypes(self):
        return ('Folder', 'Document')


class PortalFactoryXMLAdapterTests(BodyAdapterTestCase):

    def _getTargetClass(self):
        from Products.CMFPlone.exportimport.factorytool \
            import PortalFactoryXMLAdapter
        return PortalFactoryXMLAdapter

    def _populate(self, obj):
        obj.manage_setPortalFactoryTypes(listOfTypeIds=('Folder', 'Document'))

    def setUp(self):
        from Products.ATContentTypes.interfaces import IFactoryTool
        from Products.ATContentTypes.tool.factory import FactoryTool
        from Products.CMFCore.interfaces import ITypesTool
        from Products.GenericSetup.interfaces import ISetupEnviron
        from Products.GenericSetup.interfaces import IBody

        self.site = Folder('site')
        self.site.portal_types = DummyTypesTool()
        provideUtility(self.site.portal_types, ITypesTool)
        provideAdapter(self._getTargetClass(),
                       (IFactoryTool, ISetupEnviron), IBody)
        self.site.portal_factory = FactoryTool()

        self._obj = self.site.portal_factory
        self._BODY = _FACTORYTOOL_XML


def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(PortalFactoryXMLAdapterTests))
    return suite



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent
from Products.Archetypes.atapi import BooleanField
from Products.Archetypes.atapi import BooleanWidget
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import StringWidget
from Products.ATContentTypes.criteria import PATH_INDICES
from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics
from Products.CMFCore.permissions import View
from Products.CMFCore.utils import getToolByName
from zope.interface import implements


ATRelativePathCriterionSchema = ATBaseCriterionSchema + Schema((
    StringField(
        'relativePath',
        default='..',
        widget=StringWidget(
            label='Relative path',
            label_msgid="label_relativepath_criteria_customrelativepath",
            description_msgid="help_relativepath_criteria_customrelativepath",
            i18n_domain="plone",
            description=u"Enter a relative path e.g.: <br /> '..' for the "
            u"parent folder <br /> '../..' for the parent's parent <br />"
            u"'../somefolder' for a sibling folder")
    ),

    BooleanField(
        'recurse',
        mode="rw",
        write_permission=ChangeTopics,
        accessor="Recurse",
        default=False,
        widget=BooleanWidget(
            label="Search Sub-Folders",
            label_msgid="label_path_criteria_recurse",
            description="",
            description_msgid="help_path_criteria_recurse",
            i18n_domain="plone"),
    ),
))


class ATRelativePathCriterion(ATBaseCriterion):
    """A path criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATRelativePathCriterionSchema
    meta_type = 'ATRelativePathCriterion'
    archetype_name = 'Relative Path Criterion'
    shortDesc = 'Location in site relative to the current location'

    def getNavTypes(self):
        ptool = self.plone_utils
        nav_types = ptool.typesToList()
        return nav_types

    @security.protected(View)
    def getCriteriaItems(self):
        result = []
        depth = (not self.Recurse() and 1) or -1
        relPath = self.getRelativePath()

        # sanitize a bit: you never know, with all those windoze users out
        # there
        relPath = relPath.replace("\\", "/")

        # get the path to the portal object
        portalPath = list(getToolByName(
            self, 'portal_url').getPortalObject().getPhysicalPath())

        if relPath[0] == '/':
            # someone didn't enter a relative path.
            # simply use that one, relative to the portal
            path = '/'.join(portalPath) + relPath
        else:
            folders = relPath.split('/')

            # set the path to the collections path
            path = list(aq_parent(self).getPhysicalPath())

            # Now construct an absolute path based on the relative custom path.
            # Eat away from 'path' whenever we encounter a '..' in the relative
            # path.  Append all other elements other than '..'.
            for folder in folders:
                if folder == '..':
                    # chop off one level from path
                    if path == portalPath:
                        # can't chop off more
                        # just return this path and leave the loop
                        break
                    else:
                        path = path[:-1]
                elif folder == '.':
                    # don't really need this but for being complete
                    # strictly speaking some user may use a . aswell
                    pass  # do nothing
                else:
                    path.append(folder)
            path = '/'.join(path)

        if path is not '':
            result.append((self.Field(), {'query': path, 'depth': depth}))

        return tuple(result)


registerCriterion(ATRelativePathCriterion, PATH_INDICES)



# -*- coding: utf-8 -*-
from Products.GenericSetup.interfaces import IFilesystemExporter
from zope.interface import implements
from zope.interface import Interface


# TODO: This is a temporary hack to allow disabling exporting of some
# content types until all of them support proper exporting


class IDisabledExport(Interface):
    pass


class NullExporterAdapter(object):
    """Dummy exporter that does nothing
    """

    implements(IFilesystemExporter)

    def __init__(self, context):
        self.context = context

    def export(self, export_context, subdir):
        pass

    def listExportableItems(self):
        return []



# -*- coding: utf-8 -*-
from plone.app.testing.bbb import PloneTestCase
from Products.ATContentTypes.interfaces import IATContentType
from Products.CMFPlone.tests import dummy
from zope.i18nmessageid.message import Message


AddPortalTopics = 'Add portal topics'

atct_types = ('Document', 'Event', 'File', 'Folder',
              'Image', 'Link', 'News Item',
              )


class TestATContentTypes(PloneTestCase):

    def afterSetUp(self):
        perms = self.getPermissionsOfRole('Member')
        self.setPermissions(perms + [AddPortalTopics], 'Member')
        self.types = self.portal.portal_types

    def getPermissionsOfRole(self, role):
        perms = self.portal.permissionsOfRole(role)
        return [p['name'] for p in perms if p['selected']]

    def construct(self, portal_type, id, folder):
        fti = self.types.getTypeInfo(portal_type)
        fti.constructInstance(folder, id=id)
        return getattr(folder, id)

    def createWithoutConstruction(self, portal_type, id, folder):
        fti = self.types.getTypeInfo(portal_type)
        # manual creation
        p = folder.manage_addProduct[fti.product]
        m = getattr(p, fti.factory)
        m(id)  # create it
        return folder._getOb(id)

    def testPortalTypeName(self):
        for pt in atct_types:
            ob = self.construct(pt, pt, self.folder)
            self.assertEqual(ob._getPortalTypeName(), pt)
            self.assertEqual(ob.portal_type, pt)
            self.assertTrue(IATContentType.providedBy(ob))


class TestContentTypes(PloneTestCase):
    # This test mirrors TestContentTypeScripts but calls the API and
    # not the skin scripts.

    def afterSetUp(self):
        perms = self.getPermissionsOfRole('Member')
        self.setPermissions(perms + [AddPortalTopics], 'Member')

    def getPermissionsOfRole(self, role):
        perms = self.portal.permissionsOfRole(role)
        return [p['name'] for p in perms if p['selected']]

    def testDocumentEdit(self):
        self.folder.invokeFactory('Document', id='doc')
        self.folder.doc.edit(title='Foo', text='data', text_format='html')
        self.assertEqual(self.folder.doc.EditableBody(), 'data')
        self.assertEqual(self.folder.doc.Format(), 'text/html')
        self.assertEqual(self.folder.doc.Title(), 'Foo')

    def testEventEdit(self):
        self.folder.invokeFactory('Event', id='event')
        self.folder.event.edit(title='Foo',
                               start_date='2003-09-18',
                               end_date='2003-09-19')
        self.assertEqual(self.folder.event.Title(), 'Foo')
        self.assertTrue(self.folder.event.start().ISO8601()
                            .startswith('2003-09-18T00:00:00'))
        self.assertTrue(self.folder.event.end().ISO8601()
                            .startswith('2003-09-19T00:00:00'))

    def testFileEdit(self):
        self.folder.invokeFactory('File', id='file')
        self.folder.file.edit(file=dummy.File())
        self.assertEqual(str(self.folder.file), dummy.TEXT)

    def testImageEdit(self):
        self.folder.invokeFactory('Image', id='image')
        self.folder.image.edit(file=dummy.Image())
        self.assertEqual(str(self.folder.image.data), dummy.GIF)

    def testFolderEdit(self):
        self.folder.invokeFactory('Folder', id='folder')
        self.folder.folder.edit(title='Foo', description='Bar')
        self.assertEqual(self.folder.folder.Title(), 'Foo')
        self.assertEqual(self.folder.folder.Description(), 'Bar')
        # Edit a second time
        self.folder.folder.edit(title='Fred', description='BamBam')
        self.assertEqual(self.folder.folder.Title(), 'Fred')
        self.assertEqual(self.folder.folder.Description(), 'BamBam')

    def testLinkEdit(self):
        self.folder.invokeFactory('Link', id='link')
        self.folder.link.edit(remote_url='http://foo.com', title='Foo')
        self.assertEqual(self.folder.link.Title(), 'Foo')
        self.assertEqual(self.folder.link.getRemoteUrl(), 'http://foo.com')

    def testNewsItemEdit(self):
        self.folder.invokeFactory('News Item', id='newsitem')
        self.folder.newsitem.edit(text='data', text_format='html', title='Foo')
        self.assertEqual(self.folder.newsitem.EditableBody(), 'data')
        self.assertEqual(self.folder.newsitem.Format(), 'text/html')
        self.assertEqual(self.folder.newsitem.Title(), 'Foo')

    def testTopicEdit(self):
        self.portal.portal_types.Topic.global_allow = True
        self.folder.invokeFactory('Topic', id='topic')
        self.folder.topic.edit(title='Foo')
        self.assertEqual(self.folder.topic.Title(), 'Foo')


class TestContentTypeInformation(PloneTestCase):

    def afterSetUp(self):
        self.types = self.portal.portal_types

    def testTypeTitlesAreMessages(self):
        for t in self.types.values():
            # If the title is empty we get back the id
            if t.title:
                self.assertTrue(isinstance(t.Title(), Message))
            # Descriptions may be blank. Only check if there's a value.
            if t.description:
                self.assertTrue(isinstance(t.Description(), Message))



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from ExtensionClass import Base
from OFS.SimpleItem import SimpleItem
from Persistence import Persistent
from Products.Archetypes.atapi import DisplayList
from Products.ATContentTypes.config import TOOLNAME
from Products.ATContentTypes.criteria import _criterionRegistry
from Products.ATContentTypes.interfaces import IATCTTopicsTool
from Products.CMFCore.permissions import ManagePortal
from Products.CMFCore.utils import getToolByName
from zope.interface import implements


class TopicIndex(SimpleItem, Persistent):

    def __init__(self, index, friendlyName='',
                 description='', enabled=False, criteria=()):
        self.index = index
        self.friendlyName = friendlyName
        self.description = description
        self.enabled = enabled
        self.criteria = tuple(criteria)


class ATTopicsTool(Base):
    """This tool makes it possible to manage the indexes that are used
       inside topics and allows you to enter a friendly name instead of
       cryptic indexes.
    """
    implements(IATCTTopicsTool)

    security = ClassSecurityInfo()

    # This is used in ATPortalTypeCriterion to control which types are allowed
    # to search with until this is fixed in CMF or whatever, this is the way to
    # go.
    allowed_portal_types = []

    id = TOOLNAME
    meta_type = 'ATTopics Tool'
    title = 'ATTopics Tool'
    plone_tool = True

    def __init__(self):
        self.topic_indexes = {}
        self.topic_metadata = {}
        self.allowed_portal_types = []

    def getCriteriaForIndex(self, index, as_dict=False):
        """ Returns the valid criteria for a given index """
        catalog_tool = getToolByName(self, 'portal_catalog')
        try:
            indexObj = catalog_tool.Indexes[index]
        except KeyError:
            return ()
        criteria = tuple(
            _criterionRegistry.criteriaByIndex(indexObj.meta_type))
        search_criteria = _criterionRegistry.listSearchTypes()
        if as_dict:
            criteria = [{
                'name': a,
                'description': _criterionRegistry[a].shortDesc,
                } for a in criteria if a in search_criteria]
        else:
            criteria = [a for a in criteria if a in search_criteria]
        criteria.sort()
        return criteria

    @security.protected(ManagePortal)
    def addIndex(self, index, friendlyName='', description='',
                 enabled=False, criteria=None):
        """ Add a new index along with descriptive information to the index
            registry """
        if criteria is None:
            criteria = self.getCriteriaForIndex(index)
        if index in self.topic_indexes:
            objIndex = self.topic_indexes[index]
            objIndex.friendlyName = friendlyName
            objIndex.description = description
            objIndex.enabled = enabled
            objIndex.criteria = tuple(criteria)
        else:
            objIndex = TopicIndex(index, friendlyName,
                                  description, enabled, criteria)

        self.topic_indexes[index] = objIndex
        self._p_changed = True

    @security.protected(ManagePortal)
    def addMetadata(self, metadata, friendlyName='',
                    description='', enabled=False):
        """ Add a new metadata field along with descriptive information to the
            metadata registry """
        if metadata in self.topic_metadata:
            objMeta = self.topic_metadata[metadata]
            objMeta.friendlyName = friendlyName
            objMeta.description = description
            objMeta.enabled = enabled
        else:
            objMeta = TopicIndex(metadata, friendlyName, description, enabled)

        self.topic_metadata[metadata] = objMeta
        self._p_changed = True

    @security.protected(ManagePortal)
    def updateIndex(self, index, friendlyName=None,
                    description=None, enabled=None, criteria=None):
        """ Updates an existing index in the registry, unrecognized values are
            added """
        indexes = self.topic_indexes
        if friendlyName is None:
            friendlyName = indexes[index].friendlyName
        if description is None:
            description = indexes[index].description
        if enabled is None:
            enabled = indexes[index].enabled
        if criteria is None:
            criteria = indexes[index].criteria

        self.addIndex(index, friendlyName, description, enabled, criteria)

    @security.protected(ManagePortal)
    def updateMetadata(self, metadata, friendlyName=None,
                       description=None, enabled=None):
        """ Updates an existing metadata field in the registry, unrecognized values are
            added """
        meta = self.topic_metadata
        if friendlyName is None:
            friendlyName = meta[metadata].friendlyName
        if description is None:
            description = meta[metadata].description
        if enabled is None:
            enabled = meta[metadata].enabled
        self.addMetadata(metadata, friendlyName, description, enabled)

    @security.protected(ManagePortal)
    def removeIndex(self, index):
        """ Removes an existing index from the registry """
        if index in self.topic_indexes:
            del self.topic_indexes[index]
            self._p_changed = True

    @security.protected(ManagePortal)
    def removeMetadata(self, metadata):
        """ Removes an existing metadata field from the registry """
        if metadata in self.topic_metadata:
            del self.topic_metadata[metadata]
            self._p_changed = True

    @security.protected(ManagePortal)
    def createInitialIndexes(self):
        """ create indexes for all indexes in the catalog """
        indexes = self.listCatalogFields()
        for i in indexes:
            if i not in self.topic_indexes:
                enabled = False
                self.addIndex(i, friendlyName='', enabled=enabled)
        return True

    @security.protected(ManagePortal)
    def createInitialMetadata(self):
        """ create metadata for all indexes in the catalog """
        metas = self.listCatalogMetadata()
        for i in metas:
            if i not in self.topic_metadata:
                enabled = False
                self.addMetadata(i, friendlyName='', enabled=enabled)
        return True

    @security.private
    def listCatalogFields(self):
        """ Return a list of fields from portal_catalog. """
        pcatalog = getToolByName(self, 'portal_catalog')
        available = pcatalog.indexes()
        val = sorted([field for field in available])
        return val

    @security.private
    def listCatalogMetadata(self):
        """ Return a list of columns from portal_catalog. """
        pcatalog = getToolByName(self, 'portal_catalog')
        available = pcatalog.schema()
        val = sorted([field for field in available])
        return val

    def getAllPortalTypes(self):
        """ returns a list of (id, title)-tuples for each type """
        types_tool = getToolByName(self, 'portal_types')
        types = types_tool.listTypeInfo()

        all_types = [(t.id, t.title or t.id) for t in types]
        return all_types

    def getAllowedPortalTypes(self, populate_for_end_usage=1):
        """ Return all portal_types as an (id,title) tuple that are allowed
            to search with """
        all_types = self.getAllPortalTypes()

        if populate_for_end_usage == 0:
            # return whatever is in self.allowed_portal_types and make it a
            # (id, title) tuple
            return [t for t in all_types if t[0] in self.allowed_portal_types]

        if self.allowed_portal_types == []:
            filtered_types = all_types
        else:
            filtered_types = [type for type in all_types if type[
                0] in self.allowed_portal_types]

        return filtered_types

    def getEnabledIndexes(self):
        """ Returns all TopicIndex objects for enabled indexes """
        indexes = self.topic_indexes
        results = [i for i in indexes.values() if i.enabled]
        return results

    def getEnabledMetadata(self):
        """ Returns all TopicIndex objects for enabled metadata """
        meta = self.topic_metadata
        results = [i for i in meta.values() if i.enabled]
        return results

    def getIndexDisplay(self, enabled=True):
        """ Return DisplayList of Indexes and their friendly names """
        if enabled:
            index_names = self.getIndexes(True)
        else:
            index_names = self.getIndexes()
        index_dict = self.topic_indexes
        indexes = [index_dict[i] for i in index_names]
        field_list = [(f.index, f.friendlyName or f.index) for f in indexes]
        return DisplayList(field_list)

    def getMetadataDisplay(self, enabled=True):
        """ Return DisplayList of Metadata and their friendly names """
        if enabled:
            meta_names = self.getAllMetadata(True)
        else:
            meta_names = self.getAllMetadata()
        meta_dict = self.topic_metadata
        meta = [meta_dict[i] for i in meta_names]
        field_list = [(f.index, f.friendlyName or f.index) for f in meta]
        return DisplayList(field_list)

    def getEnabledFields(self):
        """ Returns a list of tuples containing the index name, friendly name,
            and description for each enabled index. """
        enabledIndexes = self.getEnabledIndexes()
        dec_fields = sorted([(i.friendlyName.lower() or
                              i.index.lower(), i.index, i.friendlyName or
                              i.index, i.description) for i in enabledIndexes])
        fields = [(a[1], a[2], a[3]) for a in dec_fields]
        return fields

    def getFriendlyName(self, index):
        """ Returns the friendly name for a given index name, or the given
            index if the firendlyname is empty or the index is not recognized
        """
        if index in self.topic_indexes:
            return self.getIndex(index).friendlyName or index
        else:
            return index

    @security.protected(ManagePortal)
    def getIndexes(self, enabledOnly=False):
        """ Returns the full list of available indexes, optionally filtering
            out those that are not marked enabled """
        if enabledOnly:
            indexes_dec = [(i.index.lower(), i.index)
                           for i in self.getEnabledIndexes()]
        else:
            # update in case of new catalog indexes
            self.createInitialIndexes()
            indexes_dec = [(i.lower(), i) for i in self.topic_indexes.keys()]

        indexes_dec.sort()
        indexes = [i[1] for i in indexes_dec]
        return indexes

    @security.protected(ManagePortal)
    def getAllMetadata(self, enabledOnly=False):
        """ Returns the full list of available metadata fields, optionally
            filtering out those that are not marked enabled """
        if enabledOnly:
            meta_dec = [(i.index.lower(), i.index)
                        for i in self.getEnabledMetadata()]
        else:
            # update in case of new catalog indexes
            self.createInitialMetadata()
            meta_dec = [(i.lower(), i) for i in self.topic_metadata.keys()]

        meta_dec.sort()
        metadata = [i[1] for i in meta_dec]
        return metadata

    def getIndex(self, index):
        """ Returns the TopicIndex object for a given index name """
        if index in self.topic_indexes:
            return self.topic_indexes[index]
        else:
            raise AttributeError('Index ' + str(index) + ' not found')

    def getMetadata(self, metadata):
        """ Returns the TopicIndex object for a given metadata name """
        if metadata in self.topic_metadata:
            return self.topic_metadata[metadata]
        else:
            raise AttributeError('Metadata ' + str(metadata) + ' not found')

    @security.protected(ManagePortal)
    def manage_saveTopicSetup(self, REQUEST=None):
        """ Set indexes and metadata from form """
        if REQUEST is None:
            return 'Nothing saved.'

        data = REQUEST.get('index', [])
        for index in data:
            enabled = 'enabled' in index
            criteria = index.get('criteria', ())
            self.updateIndex(index['index'], index['friendlyName'], index[
                             'description'], enabled, criteria)

        meta = REQUEST.get('metadata', [])
        for metadata in meta:
            enabled = 'enabled' in metadata
            self.updateMetadata(
                metadata['index'],
                metadata['friendlyName'],
                metadata['description'],
                enabled)
        return True

    @security.protected(ManagePortal)
    def manage_saveTopicSetupTypes(self, REQUEST=None):
        """ Set portal types from form """
        if REQUEST is None:
            return 'Nothing saved.'

        self.allowed_portal_types = REQUEST.get('allowed_types', [])
        return True

InitializeClass(ATTopicsTool)



# -*- coding: utf-8 -*-
from AccessControl import getSecurityManager
from AccessControl import Permissions
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from Products.CMFCore.permissions import AddPortalContent
from Products.CMFCore.permissions import ModifyPortalContent
from Products.CMFPlone.tests import PloneTestCase
from Products.PluggableAuthService.interfaces.plugins import IChallengePlugin

import urlparse


def sortTuple(t):
    l = sorted(t)
    return tuple(l)

ADD_DOC_PERM = 'ATContentTypes: Add Document'


class TestPortalFactory(PloneTestCase.PloneTestCase):

    def afterSetUp(self):
        self.membership = self.portal.portal_membership
        self.membership.addMember('member', 'secret', ['Member'], [])
        self.membership.addMember('manager', 'secret', ['Manager'], [])

    def testTraverse(self):
        temp_doc = self.folder.restrictedTraverse(
            'portal_factory/Document/tmp_id')
        self.assertEqual(temp_doc.portal_type, 'Document')
        self.assertEqual(temp_doc.getId(), 'tmp_id')

    def testTraverseEditView(self):
        edit_view = self.folder.restrictedTraverse(
            'portal_factory/Document/tmp_id/edit')
        self.assertEqual('tmp_id', edit_view.im_self.getId())
        self.assertEqual('Document', edit_view.im_self.portal_type)

    def testTraverseTwiceByDifferentContentTypes(self):
        temp_doc = self.folder.restrictedTraverse(
            'portal_factory/Document/tmp_id')
        self.assertEqual(temp_doc.portal_type, 'Document')
        self.assertEqual(temp_doc.getId(), 'tmp_id')
        temp_img = self.folder.restrictedTraverse(
            'portal_factory/Image/tmp_id_image')
        self.assertEqual(temp_img.portal_type, 'Image')
        self.assertEqual(temp_img.getId(), 'tmp_id_image')

    def testTempFolderLocalRoles(self):
        # Temporary objects should "inherit" local roles from container
        member = self.membership.getMemberById('member')
        self.portal.acl_users.addRole('Foo')

        self.folder.manage_addLocalRoles('member', ('Foo',))
        self.folder.invokeFactory('Folder', id='folder2')
        self.folder.folder2.manage_addLocalRoles('member', ('Reviewer',))

        self.assertEqual(sortTuple(member.getRolesInContext(self.folder)),
                         ('Authenticated', 'Foo', 'Member'))

        temp_object = self.folder.restrictedTraverse(
            'portal_factory/Document/tmp_id')
        self.assertEqual(sortTuple(member.getRolesInContext(temp_object)),
                         ('Authenticated', 'Foo', 'Member'))

        temp_object2 = self.folder.folder2.restrictedTraverse(
            'portal_factory/Document/tmp_id')
        self.assertEqual(sortTuple(member.getRolesInContext(temp_object2)),
                         ('Authenticated', 'Foo', 'Member', 'Reviewer'))

    def testTempFolderLocalRolesWithBlocking(self):
        # Temporary objects should "inherit" local roles from container,
        # but also need to respect PLIP 16 local role blocking
        member = self.membership.getMemberById('member')
        self.portal.acl_users.addRole('Foo')

        self.folder.manage_addLocalRoles('member', ('Foo',))
        self.folder.invokeFactory('Folder', id='folder2')
        self.folder.folder2.manage_addLocalRoles('member', ('Reviewer',))
        # make folder2 not inherit local roles
        self.portal.plone_utils.acquireLocalRoles(self.folder.folder2,
                                                  status=0)

        self.assertEqual(
            sortTuple(member.getRolesInContext(self.folder.folder2)),
            ('Authenticated', 'Member', 'Reviewer'))

        temp_object2 = self.folder.folder2.restrictedTraverse(
            'portal_factory/Document/tmp_id')
        self.assertEqual(sortTuple(member.getRolesInContext(temp_object2)),
                         ('Authenticated', 'Member', 'Reviewer'))

    def testTempObjectLocalRolesBug(self):
        # Evil monkey patch should not change all objects of a class
        self.createMemberarea('member')
        member = self.membership.getMemberById('member')
        # Make an unrelated non-temporary object for comparison
        self.login('manager')
        # BBB: we you launch this test suite with the others you need to reset
        # global_allow to True
        self.portal.portal_types.Document.manage_changeProperties(
            global_allow=1)
        self.portal.invokeFactory('Document', id='nontmp_id')
        nontemp_object = getattr(self.portal, 'nontmp_id')

        # Assume identify of the ordinary member
        self.login('member')
        folder = self.membership.getHomeFolder()
        temp_object = \
            folder.restrictedTraverse('portal_factory/Document/tmp_id')

        # Make sure member is owner of temporary object
        self.assertEqual(sortTuple(member.getRolesInContext(temp_object)),
                         ('Authenticated', 'Member', 'Owner'))
        self.assertEqual(temp_object.Creator(), 'member')

        # Make sure member is not owner of non-temporary object
        # (i.e. make sure our evil monkey patch of the temporary instance has
        # not resulted in our patching all instances of the class)
        self.assertEqual(sortTuple(member.getRolesInContext(nontemp_object)),
                         ('Authenticated', 'Member'))

    def testTempFolderPermissions(self):
        # TempFolder should "inherit" permission mappings from container
        previous_roles = \
            [r for r in self.folder.rolesOfPermission(AddPortalContent)
             if r['name'] == 'Anonymous']
        self.folder.manage_permission(AddPortalContent, ['Anonymous'], 1)
        new_roles = [r for r in self.folder.rolesOfPermission(AddPortalContent)
                     if r['name'] == 'Anonymous']
        self.assertNotEqual(previous_roles, new_roles)

        temp_folder = self.folder.restrictedTraverse(
            'portal_factory/Document/tmp_id').aq_parent
        temp_roles = \
            [r for r in temp_folder.rolesOfPermission(AddPortalContent)
             if r['name'] == 'Anonymous']

        self.assertEqual(temp_roles, new_roles)


class TestCreateObject(PloneTestCase.PloneTestCase):

    def testCreateObjectByDoCreate(self):
        # doCreate should create the real object
        temp_object = \
            self.folder.restrictedTraverse('portal_factory/Document/tmp_id')
        foo = temp_object.portal_factory.doCreate(temp_object, 'foo')
        self.assertTrue('foo' in self.folder)
        self.assertEqual(foo.get_local_roles_for_userid(TEST_USER_ID),
                         ('Owner',))

    def testUnauthorizedToCreateObjectByDoCreate(self):
        # Anonymous should not be able to create the (real) object
        # Note that Anonymous used to be able to create the temp object...
        temp_object = \
            self.folder.restrictedTraverse('portal_factory/Document/tmp_id')
        self.logout()
        self.assertRaises(ValueError, temp_object.portal_factory.doCreate,
                          temp_object, 'foo')

    def testCopyPermission(self):
        self.setRoles(['Manager'])
        self.portal.invokeFactory('Folder', id='folder_to_copy')

        pm = self.portal.portal_membership
        pm.addMember('editor', 'secret', ['Editor'], [])
        self.login('editor')
        member = pm.getMemberById('editor')
        self.assertTrue(member.checkPermission(Permissions.copy_or_move,
                                               self.portal))
        security = getSecurityManager()
        self.assertTrue(security.validate(
            self.portal, self.portal, 'manage_copyObjects'))

    def testRenamePermission(self):
        self.setRoles(['Manager'])
        self.portal.invokeFactory('Folder', id='folder_to_copy')

        pm = self.portal.portal_membership
        pm.addMember('editor', 'secret', ['Editor'], [])
        self.login('editor')
        member = pm.getMemberById('editor')
        self.assertTrue(member.checkPermission(ModifyPortalContent,
                                               self.portal))
        security = getSecurityManager()
        self.assertTrue(security.validate(
            self.portal, self.portal, 'manage_renameObjects'))


class TestCreateObjectByURL(PloneTestCase.FunctionalTestCase):
    '''Weeee, functional tests'''

    def afterSetUp(self):
        self.folder_url = self.folder.absolute_url()
        self.folder_path = '/%s' % self.folder.absolute_url(1)
        self.basic_auth = '%s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD)
        # We want 401 responses, not redirects to a login page
        plugins = self.portal.acl_users.plugins
        try:
            plugins.deactivatePlugin(IChallengePlugin,
                                     'credentials_cookie_auth')
        except KeyError:
            pass

        # Enable portal_factory for Document type
        self.factory = self.portal.portal_factory
        self.factory.manage_setPortalFactoryTypes(listOfTypeIds=['Document'])

    def testCreateObject(self):
        # createObject script should make a temp object
        response = self.publish(self.folder_path +
                                '/createObject?type_name=Document',
                                self.basic_auth)

        # Redirect to document edit form
        self.assertEqual(response.getStatus(), 302)

        # The redirect URL should contain the factory parts
        location = response.getHeader('Location')
        self.assertTrue(location.startswith(
            self.folder_url + '/portal_factory/Document/'))
        # CMFFormController redirects should not do alias translation
        self.assertTrue('/edit' in location)

        # Perform the redirect
        edit_form_path = location[len(self.app.REQUEST.SERVER_URL):]
        response = self.publish(edit_form_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def testCreateNonGloballyAllowedObject(self):
        # TempFolder allows to create all portal types
        self.portal.portal_types.Document.manage_changeProperties(
            global_allow=0)
        response = self.publish(self.folder_path +
                                '/createObject?type_name=Document',
                                self.basic_auth)

        # Redirect to document edit form
        self.assertEqual(response.getStatus(), 302)

        # The redirect URL should contain the factory parts
        location = response.getHeader('Location')
        self.assertTrue(location.startswith(
            self.folder_url + '/portal_factory/Document/'))
        self.assertTrue('/edit' in location)

        # Perform the redirect
        edit_form_path = location[len(self.app.REQUEST.SERVER_URL):]
        response = self.publish(edit_form_path, self.basic_auth)
        self.assertEqual(response.getStatus(), 200)  # OK

    def testUnauthorizedToViewEditForm(self):
        # Anonymous should not be able to see document edit form
        response = self.publish(self.folder_path +
                                '/createObject?type_name=Document',
                                )  # No basic out info
        # We got redirected to the factory
        self.assertEqual(response.getStatus(), 302)
        newpath = response.getHeader('location')
        proto, host, path, query, fragment = urlparse.urlsplit(newpath)
        # Let's follow it
        response = self.publish(path)
        # And we are forbidden
        self.assertEqual(response.getStatus(), 401)  # Unauthorized

    def testUnauthorizedToViewEditFormOfNonFactoryObject(self):
        # Anonymous should not be able to see newsitem_edit_form
        response = self.publish(self.folder_path +
                                '/createObject?type_name=News%20Item',
                                )  # No basic out info

        self.assertEqual(response.getStatus(), 401)  # Unauthorized


class TestPortalFactoryTraverseByURL(PloneTestCase.FunctionalTestCase):
    '''Weeee, functional tests'''

    def afterSetUp(self):
        self.folder_url = self.folder.absolute_url()
        self.folder_path = '/%s' % self.folder.absolute_url(1)
        self.basic_auth = '%s:%s' % (TEST_USER_NAME, TEST_USER_PASSWORD)
        # We want 401 responses, not redirects to a login page
        plugins = self.portal.acl_users.plugins
        try:
            plugins.deactivatePlugin(IChallengePlugin,
                                     'credentials_cookie_auth')
        except KeyError:
            pass

        # Enable portal_factory for Document type
        self.factory = self.portal.portal_factory
        self.factory.manage_setPortalFactoryTypes(listOfTypeIds=['Document'])

        # setup a temp object
        response = self.publish(self.folder_path +
                                '/createObject?type_name=Document',
                                self.basic_auth
                                )
        # We got redirected to the factory
        self.assertEqual(response.getStatus(), 302)
        newpath = response.getHeader('location')
        proto, host, path, query, fragment = urlparse.urlsplit(newpath)

        self.tmp_obj_path = path.replace('/edit', '')

    def testFSImage(self):
        path = "%s/logo.png" % self.tmp_obj_path
        data = self.publish(path)
        self.assertEqual(data.getHeader('Content-Type'), 'image/png')

    def testBrowserResource(self):
        path = "%s/++resource++plone-logo.png" % self.tmp_obj_path
        data = self.publish(path)
        self.assertEqual(data.getHeader('Content-Type'), 'image/png')



# -*- coding: utf-8 -*-
from plone.app.testing.bbb import PloneTestCase


class TestControlPanel(PloneTestCase):

    def afterSetUp(self):
        self.controlpanel = self.portal.portal_controlpanel

        # get the expected configlets
        self.configlets = ['portal_atct']

    def testDefaultConfiglets(self):
        for title in self.configlets:
            self.assertTrue(
                title in [a.getAction(self)['id']
                          for a in self.controlpanel.listActions()],
                "Missing configlet with id '%s'" % title)



# -*- coding: utf-8 -*-
import Products.ATContentTypes.content.link  # noqa
import Products.ATContentTypes.content.image  # noqa
import Products.ATContentTypes.content.document  # noqa
import Products.ATContentTypes.content.file  # noqa
import Products.ATContentTypes.content.event  # noqa
import Products.ATContentTypes.content.newsitem  # noqa
import Products.ATContentTypes.content.folder  # noqa
import Products.ATContentTypes.content.topic  # noqa



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from App.class_init import InitializeClass
from Products.Archetypes.atapi import BaseContentMixin
from Products.Archetypes.ClassGen import generateClass
from Products.Archetypes.interfaces.referenceable import IReferenceable
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicCriterion
from Products.CMFCore.permissions import View
from zope.interface import classImplementsOnly
from zope.interface import implementedBy
from zope.interface import implements


class NonRefCatalogContent(BaseContentMixin):
    """Base class for content that is neither referenceable nor in the catalog
    """
    isReferenceable = None

    # reference register / unregister methods
    def _register(self, *args, **kwargs): pass

    def _unregister(self, *args, **kwargs): pass

    def _updateCatalog(self, *args, **kwargs): pass

    def _referenceApply(self, *args, **kwargs): pass

    def _uncatalogUID(self, *args, **kwargs): pass

    def _uncatalogRefs(self, *args, **kwargs): pass

    # catalog methods
    def indexObject(self, *args, **kwargs): pass

    def unindexObject(self, *args, **kwargs): pass

    def reindexObject(self, *args, **kwargs): pass

classImplementsOnly(NonRefCatalogContent,
                    *(iface for iface in implementedBy(NonRefCatalogContent)
                      if iface is not IReferenceable))


class ATBaseCriterion(NonRefCatalogContent):
    """A basic criterion"""

    security = ClassSecurityInfo()

    implements(IATTopicCriterion)

    schema = ATBaseCriterionSchema
    meta_type = 'ATBaseCriterion'
    archetype_name = 'Base Criterion'

    def __init__(self, id=None, field=None, oid=None):
        if oid is not None:
            if field is None:
                field = id
            id = oid
        assert id
        NonRefCatalogContent.__init__(self, id)
        if not self.id == id:
            self.id = id
        self.getField('field').set(self, field)

    @security.protected(View)
    def getId(self):
        """Get the object id"""
        return str(self.id)

    def setId(self, value, *kw):
        """Setting a new ID isn't allowed
        """
        assert value == self.getId(), 'You are not allowed to change the id'

    @security.protected(View)
    def Type(self):
        return self.archetype_name

    @security.protected(View)
    def Description(self):
        lines = [line.strip() for line in self.__doc__.splitlines()]
        return ' '.join([line for line in lines if line])

    @security.protected(View)
    def getCriteriaItems(self):
        """Return a sequence of items to be used to build the catalog query.
        """
        return ()

# because I don't register the class I've to generator it on my own. Otherwise
# I'm not able to unit test it in the right way.
generateClass(ATBaseCriterion)
InitializeClass(ATBaseCriterion)



# -*- coding: utf-8 -*-
from zope.interface import Attribute
from zope.interface import Interface


class IFactoryTool(Interface):
    """This tool manages the portal factory type registration.
    """

    id = Attribute('id', 'Must be set to "portal_factory"')

    def getFactoryTypes():
        """Return the list of factory types which use the portal factory.
        """



# -*- coding: utf-8 -*-
from Acquisition import aq_base
from plone.app.layout.nextprevious.interfaces import INextPreviousProvider
from plone.memoize.instance import memoize
from plone.registry.interfaces import IRegistry
from Products.ATContentTypes.interfaces.folder import IATFolder
from Products.CMFCore.utils import getToolByName
from zope.component import adapts
from zope.component import getUtility
from zope.interface import implements


class ATFolderNextPrevious(object):
    """Let a folder act as a next/previous provider. This will be
    automatically found by the @@plone_nextprevious_view and viewlet.
    """

    implements(INextPreviousProvider)
    adapts(IATFolder)

    def __init__(self, context):
        self.context = context
        registry = getUtility(IRegistry)
        self.view_action_types = registry.get(
            'plone.types_use_view_action_in_listings', [])

    def getNextItem(self, obj):
        relatives = self.itemRelatives(obj.getId())
        return relatives["next"]

    def getPreviousItem(self, obj):
        relatives = self.itemRelatives(obj.getId())
        return relatives["previous"]

    @property
    def enabled(self):
        return self.context.getNextPreviousEnabled()

    @memoize
    def itemRelatives(self, oid):
        """Get the relative next and previous items
        """
        folder = self.context
        catalog = getToolByName(self.context, 'portal_catalog')
        position = folder.getObjectPosition(oid)

        previous = None
        next = None

        # Get the previous item
        if position - 1 >= 0:
            prev_brain = catalog(self.buildNextPreviousQuery(
                position=position - 1, range='max', sort_order='reverse'))
            if prev_brain and len(prev_brain) > 0:
                previous = self.buildNextPreviousItem(prev_brain[0])

        # Get the next item
        counter = getattr(aq_base(folder), 'objectCount', None)
        if counter is not None:
            count = counter()
        else:
            count = len(folder)

        if (position + 1) < count:
            next_brain = catalog(self.buildNextPreviousQuery(
                position=position + 1, range='min'))

            if next_brain and len(next_brain) > 0:
                next = self.buildNextPreviousItem(next_brain[0])

        nextPrevious = {
            'next': next,
            'previous': previous,
        }

        return nextPrevious

    def buildNextPreviousQuery(self, position, range, sort_order=None):
        sort_on = 'getObjPositionInParent'

        query = {}
        query['sort_on'] = sort_on
        query['sort_limit'] = 1
        query['path'] = dict(query='/'.join(self.context.getPhysicalPath()),
                             depth=1)

        # Query the position using a range
        if position == 0:
            query[sort_on] = 0
        else:
            query[sort_on] = dict(query=position, range=range)

        # Filters on content
        query['is_default_page'] = False
        query['is_folderish'] = False

        # Should I sort in any special way ?
        if sort_order:
            query['sort_order'] = sort_order

        return query

    def buildNextPreviousItem(self, brain):
        return {'id': brain.getId,
                'url': self.getViewUrl(brain),
                'title': brain.Title,
                'description': brain.Description,
                'portal_type': brain.portal_type,
                }

    def getViewUrl(self, brain):
        """create link and support contents that requires /view
        """
        item_url = brain.getURL()

        if brain.portal_type in self.view_action_types:
            item_url += '/view'

        return item_url



# -*- coding: utf-8 -*-
from plone.i18n.normalizer.interfaces import IFileNameNormalizer
from Products.Archetypes.event import ObjectInitializedEvent
from Products.Archetypes.interfaces import IATCTFileFactory
from Products.CMFCore.interfaces._content import IFolderish
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import utils as ploneutils
from thread import allocate_lock
from zope.component import adapts
from zope.component import getUtility
from zope.container.interfaces import INameChooser
from zope.event import notify
from zope.interface import implements
from zope.lifecycleevent import ObjectModifiedEvent

import transaction


upload_lock = allocate_lock()


class ATCTFileFactory(object):
    """
    ripped out of collective.uploadify
    """
    implements(IATCTFileFactory)
    adapts(IFolderish)

    def __init__(self, context):
        self.context = context

    def __call__(self, name, content_type, data):
        ctr = getToolByName(self.context, 'content_type_registry')
        type_ = ctr.findTypeName(name.lower(), '', '') or 'File'

        # XXX: quick fix for german umlauts
        name = name.decode("utf8")

        normalizer = getUtility(IFileNameNormalizer)
        chooser = INameChooser(self.context)

        # otherwise I get ZPublisher.Conflict ConflictErrors
        # when uploading multiple files
        upload_lock.acquire()

        # this should fix #8
        newid = chooser.chooseName(normalizer.normalize(name),
                                   self.context.aq_parent)
        try:
            transaction.begin()
            obj = ploneutils._createObjectByType(type_,
                                                 self.context, newid)
            mutator = obj.getPrimaryField().getMutator(obj)
            mutator(data, content_type=content_type)
            obj.setTitle(name)
            obj.reindexObject()

            notify(ObjectInitializedEvent(obj))
            notify(ObjectModifiedEvent(obj))

            transaction.commit()
        finally:
            upload_lock.release()
        return obj



from setuptools import setup, find_packages

version = '2.2.11'

setup(name='Products.ATContentTypes',
      version=version,
      description="Default Content Types for Plone 2.1-4.3",
      long_description=(open("README.rst").read() + "\n" +
                        open("CHANGES.rst").read()),
      classifiers=[
          "Programming Language :: Python",
          "Programming Language :: Python :: 2.7",
          "Framework :: Plone",
          "Framework :: Plone :: 5.0",
      ],
      keywords='Plone Content Types',
      author='AT Content Types development team',
      author_email='plone-developers@lists.sourceforge.net',
      url='http://plone.org/',
      license='GPL',
      packages=find_packages(exclude=['ez_setup']),
      namespace_packages=['Products'],
      include_package_data=True,
      zip_safe=False,
      extras_require=dict(
          test=[
              'zope.annotation',
              'zope.testing',
              'plone.app.testing',
          ]
      ),
      install_requires=[
          'setuptools',
          'plone.i18n',
          'plone.memoize',
          'plone.app.blob',
          'plone.app.collection',
          'plone.app.folder',
          'plone.app.layout',
          'plone.app.widgets>=1.4.0dev',
          'zope.component',
          'zope.i18n',
          'zope.i18nmessageid',
          'zope.interface',
          'zope.publisher',
          'zope.tal',
          'Products.CMFPlone',
          'Products.Archetypes',
          'Products.CMFCore',
          'Products.CMFDynamicViewFTI',
          'Products.CMFFormController',
          'Products.GenericSetup',
          'Products.MimetypesRegistry',
          'Products.PortalTransforms',
          'Products.validation',
          'Acquisition',
          'DateTime',
          'ExtensionClass',
          'transaction',
          'ZConfig',
          'ZODB3',
          'Zope2',
      ],
      )



# -*- coding: utf-8 -*-
from Products.ATContentTypes.content.schemata import ATContentTypeSchema
from Products.ATContentTypes.tests import atcttestcase
from Products.validation.interfaces.IValidator import IValidationChain


tests = []


class TestBugs(atcttestcase.ATCTSiteTestCase):

    def test_wfmapping(self):
        default = ('plone_workflow',)

        mapping = {
            'Document': default,
            'Event': default,
            'File': (),
            'Folder': ('folder_workflow',),
            'Image': (),
            'Link': default,
            'News Item': default,
            'Collection': default,
        }

        for pt, wf in mapping.items():
            pwf = self.portal.portal_workflow.getChainFor(pt)
            self.assertEqual(pwf, wf, (pt, pwf, wf))

    def test_striphtmlbug(self):
        # Test for Plone tracker #4944
        self.folder.invokeFactory('Document', 'document')
        d = getattr(self.folder, 'document')
        d.setTitle("HTML end tags start with </ and end with >")
        self.assertEqual(
            d.Title(), "HTML end tags start with </ and end with >")

    def test_validation_layer_from_id_field_from_base_schema_was_initialized(
            self):
        field = ATContentTypeSchema['id']
        self.assertTrue(IValidationChain.providedBy(field.validators))



# -*- coding: utf-8 -*-
from Products.ATContentTypes.tool.factory import FactoryTool
from Products.ATContentTypes.tool.factory import TempFolder
from Products.ATContentTypes.tool.metadata import MetadataTool
from Products.CMFPlone.tests.testInterfaces import className
from Products.CMFPlone.tests.testInterfaces import InterfaceTest
from Products.CMFPlone.tests.testInterfaces import zope_interface_test

import unittest


tests = []
# format: (class object, (list interface objects))
testClasses = [
    (FactoryTool, ()),
    (TempFolder, ()),
    (MetadataTool, ()),
]

for testClass in testClasses:
    klass, forcedImpl = testClass
    name = className(klass)
    funcName = 'test%sInterface' % name

    class KlassInterfaceTest(InterfaceTest):
        """ implementation for %s """ % name
        klass = klass
        forcedImpl = forcedImpl

    # add the testing method to the class to get a nice name
    setattr(KlassInterfaceTest, funcName, lambda self: self._testStuff())
    tests.append(KlassInterfaceTest)

    class KlassInterfaceTest(zope_interface_test):
        """ implementation for %s """ % name
        klass = klass
        forcedImpl = forcedImpl

    # add the testing method to the class to get a nice name
    setattr(KlassInterfaceTest, funcName, lambda self: self._testStuff())
    tests.append(KlassInterfaceTest)


def test_suite():
    suite = unittest.TestSuite()
    for test in tests:
        suite.addTest(unittest.makeSuite(test))
    return suite



# -*- coding: utf-8 -*-
from Products.Archetypes.atapi import IdWidget
from Products.Archetypes.atapi import Schema
from Products.Archetypes.atapi import StringField
from Products.Archetypes.atapi import StringWidget
from Products.ATContentTypes import ATCTMessageFactory as _
from Products.ATContentTypes.permission import ChangeTopics


###
# AT Base Criterion
###

ATBaseCriterionSchema = Schema((
    StringField(
        'id',
        required=1,
        mode="r",
        default=None,
        write_permission=ChangeTopics,
        widget=IdWidget(
            label=_(u'label_short_name', default=u'Short Name'),
            description=_(
                u'help_shortname',
                default=u"Should not contain spaces, underscores or mixed "
                u"case. Short Name is part of the item's web address."),
            visible={'view': 'invisible'}
        ),
    ),

    StringField(
        'field',
        required=1,
        mode="r",
        accessor="Field",
        write_permission=ChangeTopics,
        default=None,
        widget=StringWidget(
            label=_(u'label_criteria_field_name',
                    default=u'Field name'),
            description=_(
                u'help_shortname',
                default=u"Should not contain spaces, underscores or mixed "
                u"case. Short Name is part of the item's web address.")
        ),
    ),
))



# -*- coding: utf-8 -*-
from AccessControl import ClassSecurityInfo
from Products.Archetypes.atapi import CalendarWidget
from Products.Archetypes.atapi import DateTimeField
from Products.Archetypes.atapi import Schema
from Products.ATContentTypes import ATCTMessageFactory as _
from Products.ATContentTypes.criteria import DATE_INDICES
from Products.ATContentTypes.criteria import registerCriterion
from Products.ATContentTypes.criteria.base import ATBaseCriterion
from Products.ATContentTypes.criteria.schemata import ATBaseCriterionSchema
from Products.ATContentTypes.interfaces import IATTopicSearchCriterion
from Products.ATContentTypes.permission import ChangeTopics
from Products.CMFCore.permissions import View
from zope.interface import implements


RELEVANT_INDICES = list(DATE_INDICES)
RELEVANT_INDICES.remove('DateRangeIndex')
RELEVANT_INDICES = tuple(RELEVANT_INDICES)

ATDateRangeCriterionSchema = ATBaseCriterionSchema + Schema((
    DateTimeField(
        'start',
        required=1,
        mode="rw",
        write_permission=ChangeTopics,
        default=None,
        widget=CalendarWidget(
            label=_(u'label_date_range_criteria_start',
                    default=u'Start Date'),
            description=_(
                u'help_date_range_criteria_start',
                default=u'The beginning of the date range to search')
        ),
    ),

    DateTimeField(
        'end',
        required=1,
        mode="rw",
        write_permission=ChangeTopics,
        default=None,
        widget=CalendarWidget(
            label=_(u'label_date_range_criteria_end',
                    default=u'End Date'),
            description=_(
                u'help_date_range_criteria_end',
                default=u'The ending of the date range to search.')

        ),
    ),
))


class ATDateRangeCriterion(ATBaseCriterion):
    """A date range criterion"""

    implements(IATTopicSearchCriterion)

    security = ClassSecurityInfo()
    schema = ATDateRangeCriterionSchema
    meta_type = 'ATDateRangeCriterion'
    archetype_name = 'Date Range Criterion'
    shortDesc = 'Date range'

    @security.protected(View)
    def Value(self):
        return (self.getStart(), self.getEnd())

    @security.protected(View)
    def getCriteriaItems(self):
        field = self.Field()
        value = self.Value()

        return ((field, {'query': value, 'range': 'min:max'}),)

registerCriterion(ATDateRangeCriterion, RELEVANT_INDICES)
